Package: BoundIRT
Title: Fit Bounded Continuous Item Response Theory Models to Data
Version: 0.6.0
Authors@R: 
    person("Dylan", "Molenaar", , "d.molenaar@uva.nl", role = c("aut", "cre"))
Description: Bounded continuous data are encountered in many areas of test application. 
 Examples include visual analogue scales used in the measurement of personality, mood, 
 depression, and quality of life; item response times from tests with item deadlines; 
 confidence ratings; and pain intensity ratings. Using this package, item response theory (IRT) 
 models suitable for bounded continuous item scores can be fitted to data within a Bayesian framework. 
 The package draws on posterior sampling facilities provided by R-package 'rstan' (Stan Development Team, 2025)<https://mc-stan.org/>. 
 Available models include the Beta IRT model by Noel and Dauvier (2007)<doi:10.1177/0146621605287691>, the continuous response 
 model by Samejima (1973)<doi:10.1007/BF03372160>, the unbounded normal model by Mellenbergh (1994)<doi:10.1207/s15327906mbr2903_2>, 
 and the Simplex IRT model by Flores et al. (2020)<doi:10.1007/978-3-030-43469-4_8>. All models can be 
 fitted with or without zero-one inflation (Molenaar et al., 2022)<doi:10.3102/10769986221108455>. 
 Model fit comparisons can be conducted using the Watanabe-Akaike information criterion (WAIC), leave-one-out 
 cross-validation information citerion (LOOIC) and the fully marginalized likelihood (i.e., Bayes factors).
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Biarch: true
Depends: R (>= 3.5)
Imports: methods, mvtnorm, MASS, statmod, Rcpp (>= 0.12.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), rstantools (>=
        2.6.0)
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
SystemRequirements: GNU make
LazyData: true
LazyDataCompression: xz
Suggests: bayesplot, loo, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-08-20 13:43:35 UTC; dmolena1
Author: Dylan Molenaar [aut, cre]
Maintainer: Dylan Molenaar <d.molenaar@uva.nl>
Repository: CRAN
Date/Publication: 2026-08-21 05:47:08 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-08-21 06:38:36 UTC; unix
Archs: BoundIRT.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(coef,BoundIRT)
S3method(plot,BoundIRT)
S3method(print,BoundIRT)
S3method(print,bridgeBIRT)
S3method(print,latregBIRT)
S3method(summary,BoundIRT)
export(bridgeBIRT)
export(fitBIRT)
export(latregBIRT)
export(loglikBIRT)
export(rBIRT)
export(repBIRT)
import(Rcpp)
import(methods)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
useDynLib(BoundIRT, .registration = TRUE)

#' @export
bridgeBIRT=function(object,niter=10000,tol=1e-7,silent=FALSE){

  inflated=object$inflated

  if(inflated=="none") output0=bridgeBIRTno(object,niter,tol,silent)
  else output0=bridgeBIRTinf(object,niter,tol,silent)

  log_ml=log(output0$r1)+output0$l_star
  output=append(output0,list(log_ml=log_ml),after=0)
  output=output[c(1,3,2)]
  class(output)="bridgeBIRT"
return(output)
}

#' @export

repBIRT=function(object,nrep=1000,silent=FALSE){
  mcmc=object$mcmc
  model=object$model
  inflated=object$inflated
  N=nrow(object$z)
  nit=ncol(object$z)
  item.out=object$output

  alpha=rstan::extract(mcmc,"alpha")$"alpha"
  nsamples0=nrow(alpha)
  if(nsamples0<nrep) stop("the nrep provided exceeds the number of actual posterior samples")
  index=rev(seq(nsamples0,nsamples0-nrep,by=-1))
  alpha=alpha[index,]
  beta=rstan::extract(mcmc,"beta")$"beta"[index,]
  phi=rstan::extract(mcmc,"phi")$"phi"[index,]
  theta=rstan::extract(mcmc,"theta")$"theta"[index,]


  if(inflated=="zero" | inflated=="both") gamma0_c=rstan::extract(mcmc,"gamma0")$"gamma0"[index,] else gamma0_c=matrix(-Inf,nrep,nit)
  if(inflated=="one" | inflated=="both") gamma1_c=rstan::extract(mcmc,"gamma1")$"gamma1"[index,] else gamma1_c=matrix(Inf,nrep,nit)


  if(inflated=="zero" | inflated=="both"){
    if(inflated=="zero"){
      one_mask=rep(0,nit)
      one_mask_matrix=diag(0,nit)
    }
    zero_mask=1*(is.finite(item.out[,"gamma0"]))
    zero_mask_matrix=matrix(0,sum(zero_mask),nit)
    for(i in 1:nit){
      if(zero_mask[i]==1) zero_mask_matrix[sum(zero_mask[1:i]),i]=1
      if(zero_mask[i]==0) zero_mask_matrix[sum(zero_mask[1:i]),i]=0
    }
  }

  if(inflated=="one" | inflated=="both"){
    if(inflated=="one"){
      zero_mask=rep(0,nit)
      zero_mask_matrix=diag(0,nit)
    }
    one_mask=1*(is.finite(item.out[,"gamma1"]))
    one_mask_matrix=matrix(0,sum(one_mask),nit)
    for(i in 1:nit){
      if(one_mask[i]==1) one_mask_matrix[sum(one_mask[1:i]),i]=1
      if(one_mask[i]==0) one_mask_matrix[sum(one_mask[1:i]),i]=0
    }
  }

  gamma0=gamma0_c%*%zero_mask_matrix
  gamma0[,zero_mask==0]=-Inf
  gamma1=gamma1_c%*%one_mask_matrix
  gamma1[,one_mask==0]=Inf

  z_rep=array(NA,dim=c(nrep,N,nit))
  for(s in 1:nrep){
    if(!silent) cat("Simulating replicated data for posterior sample: ",s,"\r")
    z_rep[s,,]=rBIRT(N,nit,alpha[s,],beta[s,],phi[s,],theta[s,],gamma0[s,],gamma1[s,],model = model)
  }
  if(!silent) cat("\n")
  return(z_rep)
}

#' @export

fitBIRT=function(z,model=c("beta","samejima","simplex","normal"),inflated=c("auto","zero","one","both","none"),iter=2000,warmup=NULL,nchains=1,prior=NULL,silent=FALSE,...){
  model=match.arg(model)
  inflated=match.arg(inflated)
  args=list(...)
  if(silent) refresh=0
  else{
    if(is.null(args$refresh)) refresh=max(iter/10, 1) else refresh=args$refresh
  }

  if(any(z>1,na.rm=TRUE) | any(z<0,na.rm=TRUE)) stop("Item scores shoud be between 0 and 1")

  if(inflated=="none") if(any(z==0,na.rm=TRUE)|any(z==1,na.rm=TRUE)) stop("There are boundary responses in your data, use zero and/or one inflation, or inflated=\"auto\"")
  if(inflated=="zero") if(any(z==1,na.rm=TRUE)) stop("You use zero inflation, but there is one inflation in your data (as well)")
  if(inflated=="one") if(any(z==0,na.rm=TRUE)) stop("You use one inflation, but there is one inflation in your data (as well)")

   N=nrow(z)
   nit=ncol(z)

   if(inflated=="zero" | inflated=="both") zero_inf=1:nit
   if(inflated=="one" | inflated=="both") one_inf=1:nit
   if(inflated=="zero" | inflated=="none") one_inf=rep(0,nit)
   if(inflated=="one" | inflated=="none") zero_inf=rep(0,nit)

   auto=FALSE
   if(inflated=="auto"){
     auto=TRUE
     zeros=apply(z,2,min,na.rm=TRUE)
     zero_inf=1*(zeros==0)

     ones=apply(z,2,max,na.rm=TRUE)
     one_inf=1*(ones==1)

     zero_inf[zero_inf==1]=1:sum(zero_inf)
     one_inf[one_inf==1]=1:sum(one_inf)
     if(max(zero_inf)==0 & max(one_inf)==0) inflated="none"
     if(max(zero_inf)==0 & max(one_inf)!=0) inflated="one"
     if(max(zero_inf)!=0 & max(one_inf)==0) inflated="zero"
     if(max(zero_inf)!=0 & max(one_inf)!=0) inflated="both"
   }

   if(is.null(prior)){
     prior=list(sT=1,sA=10,sB=10,sP=10)
     if(inflated!="none") prior$sG=10
   }
   else{
     if(!is.list(prior)) stop("'prior' should be a list")
     nn=names(prior)
     if(!("sT"%in%nn)) prior$sT=1
     if(!("sA"%in%nn)) prior$sA=10
     if(!("sB"%in%nn)) prior$sB=10
     if(!("sP"%in%nn)) prior$sP=10
     if(inflated!="none" & (!("sG"%in%nn))) prior$sG=10
    }


   par_names=c("theta","phi","alpha","beta", "gamma0","gamma1")
   if(inflated=="none") par_names=par_names[1:4]
   if(inflated=="one") par_names=par_names[c(1:4,6)]
   if(inflated=="zero") par_names=par_names[1:5]
   if(inflated=="both") par_names=par_names[1:6]

   inits=list()
   for(jj in 1:nchains) inits[[jj]]=list(
     ln_alpha=stats::rnorm(nit,0,.2),
     ln_phi=stats::rnorm(nit,0,.2),
     beta=stats::rnorm(nit,1,.5),
     theta=stats::rnorm(N,0,1),
     gamma0=-abs(stats::rnorm(max(zero_inf),1,.2)),
     gamma1=(abs(stats::rnorm(max(one_inf),1,.2))))


   if(inflated=="none") for(jj in 1:nchains) inits[[jj]]=inits[[jj]][-c(5,6)]
   if(inflated=="zero" | max(zero_inf)==1) for(jj in 1:nchains) inits[[jj]]=inits[[jj]][-c(6)]
   if(inflated=="one"| max(one_inf)==1) for(jj in 1:nchains) inits[[jj]]=inits[[jj]][-c(5)]
   if(max(zero_inf)==1) for(jj in 1:nchains) inits[[jj]]=inits[[jj]][-c(5)]
   if(max(one_inf)==1) for(jj in 1:nchains) inits[[jj]]=inits[[jj]][-c(6)]

   if(is.null(warmup)) warmup = floor(iter/2)
   z[is.na(z)]=-999

   cap_first <- function(x) {
     paste0(toupper(substr(x, 1, 1)), substr(x, 2, nchar(x)))
   }

   if(inflated=="none") input=append(list(N=N,z=z,nit=nit),prior)
   else input=append(list(N=N,z=z,nit=nit,zero_inf=zero_inf,one_inf=one_inf,n_zero=max(zero_inf),n_one=max(one_inf)),prior)

   if(!silent){
     cat("Fitting the",cap_first(model),"Bounded Continuous IRT model")
     if(inflated!="none" & inflated!="both") cat(" with",paste0(cap_first(inflated), "-inflation"))
     if(inflated=="both") cat(" with both Zero- and One-inflation")
     cat(" . . .\n\n")
     if(auto & (inflated=="zero"|inflated=="both")) cat("     Zero inflation found for items: ",which(zero_inf>0),"\n")
     if(auto & (inflated=="one"|inflated=="both"))  cat("     One inflation found for items: ",which(one_inf>0),"\n")
     cat("\n")
   }

   if(model=="samejima"){
     z0=z
     z0[z0!=0 & z0!=1 & z0!=-999]=stats::qlogis(z[z!=0 & z!=1 & z!=-999])
     input$z=z0
     stanmodels$SamejimaIRT=stanmodels$NormalIRT
     stanmodels$SamejimaIRT_inf=stanmodels$NormalIRT_inf
     if(inflated=="none") fit <- rstan::sampling(stanmodels$SamejimaIRT, iter=iter, warmup=warmup, chains=nchains,
                                                 data = input ,init=inits,pars=par_names,refresh=refresh,...)
     else fit <- rstan::sampling(stanmodels$SamejimaIRT_inf, iter=iter, warmup=warmup, chains=nchains,
                                 data = input ,init=inits,pars=par_names,refresh=refresh,...)
   }
   if(model=="beta"){
     if(inflated=="none") fit <- rstan::sampling(stanmodels$BetaIRT, iter=iter, warmup=warmup, chains=nchains,
                                                 data = input,init=inits,pars=par_names,refresh=refresh,...)
     else fit <- rstan::sampling(stanmodels$BetaIRT_inf, iter=iter, warmup=warmup, chains=nchains,
                                 data = input,init=inits,pars=par_names,refresh=refresh,...)
   }
   if(model=="simplex"){
    if(inflated=="none") fit <- rstan::sampling(stanmodels$SimplexIRT, iter=iter, warmup=warmup, chains=nchains,
                                                data = input,init=inits,pars=par_names,refresh=refresh,...)
    else fit <- rstan::sampling(stanmodels$SimplexIRT_inf, iter=iter, warmup=warmup, chains=nchains,
                                data = input,init=inits,pars=par_names,refresh=refresh,...)
   }
   if(model=="normal"){
     if(inflated=="none") fit <- rstan::sampling(stanmodels$NormalIRT, iter=iter, warmup=warmup, chains=nchains,
                                                 data = input,init=inits,pars=par_names,refresh=refresh,...)
     else fit <- rstan::sampling(stanmodels$NormalIRT_inf, iter=iter, warmup=warmup, chains=nchains,
                                 data = input,init=inits,pars=par_names,refresh=refresh,...)
    }

   theta.est=apply(rstan::extract(fit,"theta")$theta,2,mean)
   beta.est=apply(rstan::extract(fit,"beta")$beta,2,mean)
   alpha.est=apply(rstan::extract(fit,"alpha")$alpha,2,mean)
   phi.est=apply(rstan::extract(fit,"phi")$phi,2,mean)
   if(inflated=="zero" |inflated=="both"){
     gamma0_c.est=apply(rstan::extract(fit,"gamma0")$gamma0,2,mean)
     gamma0.est=zero_inf+-Inf
     gamma0.est[zero_inf>0]=gamma0_c.est
   } else gamma0.est=NULL

   if(inflated=="one"|inflated=="both"){
     gamma1_c.est=apply(rstan::extract(fit,"gamma1")$gamma1,2,mean)
     gamma1.est=one_inf+Inf
     gamma1.est[one_inf>0]=gamma1_c.est
   } else gamma1.est=NULL

   theta.sd=apply(rstan::extract(fit,"theta")$theta,2,stats::sd)
   beta.sd=apply(rstan::extract(fit,"beta")$beta,2,stats::sd)
   alpha.sd=apply(rstan::extract(fit,"alpha")$alpha,2,stats::sd)
   phi.sd=apply(rstan::extract(fit,"phi")$phi,2,stats::sd)
   if(inflated=="zero" |inflated=="both"){
     gamma0_c.sd=apply(rstan::extract(fit,"gamma0")$gamma0,2,stats::sd)
     gamma0.sd=zero_inf
     gamma0.sd[zero_inf>0]=gamma0_c.sd
   } else gamma0.sd=NULL

   if(inflated=="one" |inflated=="both"){
     gamma1_c.sd=apply(rstan::extract(fit,"gamma1")$gamma1,2,stats::sd)
     gamma1.sd=one_inf
     gamma1.sd[one_inf>0]=gamma1_c.sd
   } else gamma1.sd=NULL



   item_out=cbind(alpha.est,alpha.sd,beta.est,beta.sd,phi.est,phi.sd,gamma0.est,gamma0.sd,gamma1.est,gamma1.sd)
   colnames(item_out)[1:6]=c("alpha","sd","beta","sd","phi","sd")
   rownames(item_out)=paste("item",1:nit,sep="")
   if(inflated=="zero" |inflated=="both") colnames(item_out)[7:8]=c("gamma0","sd")
   if(inflated=="both") colnames(item_out)[9:10]=c("gamma1","sd")
   if(inflated=="one") colnames(item_out)[7:8]=c("gamma1","sd")

   theta_out=cbind(theta.est,theta.sd)
   colnames(theta_out)=c("theta_p","sd")

   total_output=list(output=item_out,theta=theta_out,mcmc=fit,z=z,model=model,inflated=inflated,prior=prior)
   class(total_output)="BoundIRT"
   return(total_output)
}

iicBIRT=function(theta,alpha,beta,phi,gamma0=NULL,gamma1=NULL,model=c("beta","samejima","simplex","normal")){
  model=match.arg(model)
  if(phi<0) stop("The dispersion parameter phi should be larger than 0")
  if(is.null(gamma0)) gamma0=-Inf
  if(is.null(gamma1)) gamma1=Inf

  if((length(beta)!=1) |   (length(alpha)!=1) | (length(phi)!=1) | (length(gamma0)!=1) | (length(gamma1)!=1))
    stop("Item parameters should be of length 1")

  if(model=="beta"){
    ln_phi=log(phi)
    a = exp(.5* (alpha*theta + beta+ln_phi));
    b  = exp(.5* (-alpha*theta - beta+ln_phi));
    info0=-alpha^2*((trigamma(a+b)*((a-b)/2)^2-trigamma(a)*(a/2)^2-trigamma(b)*(b/2)^2))
  }

  if(model=="simplex"){
    dsimplex=function (y, m, s2){  #from rmutil by Swihart & Lindsey (2022) but with minor adjustment
      tmp <- -((y - m)/(m * (1 - m)))^2/(2 * y * (1 - y) * s2) -
        (log(2 * pi * s2) + 3 * (log(y) + log(1 - y)))/2
      return(exp(tmp))
    }
    infX2=function(x,theta,beta,alpha,phi){
      mu=stats::plogis(alpha*theta+beta)
      tmp= ((x - mu) * (x * (2*mu - 1) - mu^2))/((x - 1) * x * phi * (1 - mu)^3 * mu^3) * alpha * mu * (1-mu)
      return(tmp^2*dsimplex(x,mu,phi))
    }
    info0=c()
    for(i in 1:length(theta)) info0[i]=stats::integrate(infX2,0.0001,.9999,theta=theta[i],beta=beta,alpha=alpha,phi=phi)$value
  }

  if(model=="samejima") info0=rep(alpha^2/phi,length(theta))
  if(model=="normal") info0=rep(alpha^2/phi,length(theta))

  info_full=c()
 if((gamma0!=-Inf) | (gamma1!=Inf)){
  for(i in 1:length(theta)){
    info_full[i]=alpha^2*stats::plogis(-(gamma0-alpha*theta[i]))^2*stats::plogis((gamma0-alpha*theta[i]))+                     #info lower bound
       (stats::plogis((gamma1-alpha*theta[i]))-stats::plogis((gamma0-alpha*theta[i])))*
       ((-alpha*(stats::plogis((gamma1-alpha*theta[i]))*stats::plogis(-(gamma1-alpha*theta[i]))-stats::plogis((gamma0-alpha*theta[i]))*stats::plogis(-(gamma0-alpha*theta[i])))/(stats::plogis((gamma1-alpha*theta[i]))-stats::plogis((gamma0-alpha*theta[i]))))^2+  #info 0<X<1
          info0[i]) +       # info traditional model
       alpha^2*stats::plogis((gamma1-alpha*theta[i]))^2*stats::plogis(-(gamma1-alpha*theta[i]))                       #info upper bound
  }
 } else info_full=info0

 return(info_full)
 }

#' @export
#' @method print BoundIRT
print.BoundIRT=function(x, digits=3, ...){
  xx=x$output
  output=format(round(xx,digits),justify="right")

  if(x$inflated=="zero"|x$inflated=="both"){
    ind=is.infinite(xx[,"gamma0"])
    output[ind,"gamma0"]=" "
    output[ind,8]=" "

  }
  if(x$inflated=="one"|x$inflated=="both"){
    ind=is.infinite(xx[,"gamma1"])
    output[ind,"gamma1"]=" "
    if(x$inflated=="one") output[ind,8]=" "
    if(x$inflated=="both") output[ind,10]=" "
  }
  col_width <- apply(output,2,function(x) max(nchar(x)))
  colnames(output) <- mapply(formatC, colnames(output), width = col_width, MoreArgs = list(flag = " "))
  print(output,quote=FALSE)
}

bridgeBIRTinf=function(object, niter,tol,silent){
fit=object$mcmc
z=object$z
model=object$model
z=as.matrix(z)
item.out=object$output
inflated=object$inflated

tau.samples=rstan::extract(fit,"theta")$theta
ti.samples=rstan::extract(fit,"beta")$beta
ln_phi.samples=log(rstan::extract(fit,"alpha")$alpha)
ln_sigma2.samples=log(rstan::extract(fit,"phi")$phi)
if(inflated=="zero" | inflated=="both") b0.samples=rstan::extract(fit,"gamma0")$gamma0 else b0.samples=NULL
if(inflated=="one" | inflated=="both") b1.samples=rstan::extract(fit,"gamma1")$gamma1 else b1.samples=NULL

N=nrow(z)
nit=ncol(z)
no_iter=nrow(tau.samples)


if(inflated=="zero" | inflated=="both"){
  if(inflated=="zero"){
    one_mask=rep(0,nit)
    one_mask_matrix=diag(0,nit)
  }
  zero_mask=1*(is.finite(item.out[,"gamma0"]))
  zero_mask_matrix=matrix(0,sum(zero_mask),nit)
  for(i in 1:nit){
    if(zero_mask[i]==1) zero_mask_matrix[sum(zero_mask[1:i]),i]=1
    if(zero_mask[i]==0) zero_mask_matrix[sum(zero_mask[1:i]),i]=0
  }
}

if(inflated=="one" | inflated=="both"){
  if(inflated=="one"){
    zero_mask=rep(0,nit)
    zero_mask_matrix=diag(0,nit)
  }
  one_mask=1*(is.finite(item.out[,"gamma1"]))
  one_mask_matrix=matrix(0,sum(one_mask),nit)
  for(i in 1:nit){
    if(one_mask[i]==1) one_mask_matrix[sum(one_mask[1:i]),i]=1
    if(one_mask[i]==0) one_mask_matrix[sum(one_mask[1:i]),i]=0
  }
}


tau.est=apply(tau.samples,2,mean)
ti.est=apply(ti.samples,2,mean)
ln_phi.est=apply(ln_phi.samples,2,mean)
ln_sigma2.est=apply(ln_sigma2.samples,2,mean)
if(inflated=="zero" | inflated=="both") b0.est=apply(b0.samples,2,mean)%*%zero_mask_matrix else b0.est=NULL
if(inflated=="one" | inflated=="both") b1.est=apply(b1.samples,2,mean)%*%one_mask_matrix else b1.est=NULL

b0.est[zero_mask==0]=-Inf
b1.est[one_mask==0]=Inf






###
n1=n2=floor(no_iter/2)

full.samples=cbind(tau.samples,ti.samples,ln_phi.samples,ln_sigma2.samples,b0.samples,b1.samples)
tau.ind=1:N
ti.ind=1:nit+N
ln_phi.ind=1:nit+N+nit
ln_sigma2.ind=1:nit+N+nit*2
if(inflated=="one") b0.ind=NULL else b0.ind=1:sum(zero_mask) + nit+N+nit*2
if(inflated=="zero") b1.ind=NULL else b1.ind=1:sum(one_mask) + nit+N+nit*2 + sum(zero_mask)

n_par=3*nit+sum(zero_mask)+sum(one_mask)

post.batch1=full.samples[1:n1,]
post.batch2=full.samples[(n1+1):(2*n1),]

propos.me.p=apply(post.batch1[,tau.ind],2,mean)
propos.me.i=apply(post.batch1[,(N+1):(max(c(b0.ind,b1.ind)))],2,mean)

propos.cov.p=diag(diag(stats::cov(post.batch1[,tau.ind])))
propos.cov.i=stats::cov(post.batch1[,(N+1):(max(c(b0.ind,b1.ind)))])

propos.samples.p=MASS::mvrnorm(n2,propos.me.p,propos.cov.p)
propos.samples.i=MASS::mvrnorm(n2,propos.me.i,propos.cov.i)

if(inflated=="both"){
  tmp_x=propos.samples.i[,b0.ind-N]%*%zero_mask_matrix
  tmp_x[,zero_mask==0]=-Inf
  tmp_y=propos.samples.i[,b1.ind-N]%*%one_mask_matrix
  tmp_y[,one_mask==0]=Inf

  error=which((tmp_y - tmp_x)<=0,arr.ind=T)[,1]
while(length(error>0)){
 propos.samples.i[error,]=MASS::mvrnorm(length(error),propos.me.i,propos.cov.i)
 tmp_x=propos.samples.i[,b1.ind-N]%*%zero_mask_matrix
 tmp_x[,zero_mask==0]=-Inf
 tmp_y=propos.samples.i[,b0.ind-N]%*%one_mask_matrix
 tmp_y[,one_mask==0]=Inf

 error=which((tmp_y - tmp_x)<=0,arr.ind=T)[,1]
}
}

propos.samples=cbind(propos.samples.p,propos.samples.i)

propos.me=c(propos.me.p,propos.me.i)
propos.cov=rbind(cbind(propos.cov.p,matrix(0,N,n_par)),cbind(matrix(0,n_par,N),propos.cov.i))

debug1=matrix(,n1,5)

dsimplex=function (y, m, s2){  #from rmutil by Swihart & Lindsey (2022) but with minor adjustment
  tmp <- -((y - m)/(m * (1 - m)))^2/(2 * y * (1 - y) * s2) -
    (log(2 * pi * s2) + 3 * (log(y) + log(1 - y)))/2
  return(exp(tmp))
}

log.l2=c()
if(!silent) cat("Stage 1/2:\n")
for(i in 1:n2){
  if(!silent) cat("processing proposal sample",i,"\r")
  tau.tmp2=propos.samples[i,tau.ind]
  ti.tmp2=propos.samples[i,ti.ind]
  ln_phi.tmp2=propos.samples[i,ln_phi.ind]
  ln_sigma2.tmp2=propos.samples[i,ln_sigma2.ind]
  if(inflated=="zero"|inflated=="both") b0.tmp2=propos.samples[i,b0.ind]%*%zero_mask_matrix else b0.tmp2=NULL
  if(inflated=="one"|inflated=="both") b1.tmp2=propos.samples[i,b1.ind]%*%one_mask_matrix else b1.tmp2=NULL

  b0.tmp2[zero_mask==0]=-Inf
  b1.tmp2[one_mask==0]=Inf

  ou5=ou1=ou2=ou3=ou4=c()
  for(p in 1:N){
    ind=which(z[p,]<1 & z[p,]>0 & z[p,]!=-999)
    tmp=c()
    if(model=="beta")
    tmp[ind] = stats::dbeta(z[p,ind],exp(.5*(exp(ln_phi.tmp2[ind])*tau.tmp2[p]+ti.tmp2[ind]+ln_sigma2.tmp2[ind])),exp(.5*(-exp(ln_phi.tmp2[ind])*tau.tmp2[p]-ti.tmp2[ind]+ln_sigma2.tmp2[ind])),log=T)
    if(model=="simplex")
    tmp[ind] = log(dsimplex(z[p,ind],m=1/(1+exp(-(ti.tmp2[ind]+exp(ln_phi.tmp2[ind])*tau.tmp2[p]))),s=exp(ln_sigma2.tmp2[ind])))
    if(model=="samejima")
    tmp[ind] = -log(z[p,ind])-log(1-z[p,ind])+stats::dnorm(stats::qlogis(z[p,ind]),ti.tmp2[ind]+exp(ln_phi.tmp2[ind])*tau.tmp2[p],exp(.5*ln_sigma2.tmp2[ind]),log=T)
    if(model=="normal")
    tmp[ind] = stats::dnorm(z[p,ind],ti.tmp2[ind]+exp(ln_phi.tmp2[ind])*tau.tmp2[p],exp(.5*ln_sigma2.tmp2[ind]),log=T)

    tmp[ind]=tmp[ind]+ log(  stats::plogis(b1.tmp2[ind]-exp(ln_phi.tmp2[ind])*tau.tmp2[p]) -  stats::plogis(b0.tmp2[ind]-exp(ln_phi.tmp2[ind])*tau.tmp2[p]))
    if(inflated=="zero"|inflated=="both"){
      ind=which(z[p,]==0)
      tmp[ind] =log( stats::plogis(b0.tmp2[ind]-exp(ln_phi.tmp2[ind])*tau.tmp2[p]))
    }
    if(inflated=="one"|inflated=="both"){
      ind=which(z[p,]==1)
      tmp[ind] =log( 1-stats::plogis(b1.tmp2[ind]-exp(ln_phi.tmp2[ind])*tau.tmp2[p]))
    }
    ou1[p]=sum(tmp[is.finite(tmp)],na.rm=T)
    ou4[p]=stats::dnorm(tau.tmp2[p],log=T)
  }
  ou2=sum(log(stats::dnorm(ln_phi.tmp2,0,10))+log(stats::dnorm(ln_sigma2.tmp2,0,10))+log(stats::dnorm(ti.tmp2,0,10))) + sum(log(stats::dnorm(b0.tmp2[zero_mask==1],0,10)))+sum(log(stats::dnorm(b1.tmp2[one_mask==1],0,10)/stats::pnorm(b0.tmp2[one_mask==1],0,10,low=F)))
  ou3=mvtnorm::dmvnorm(c(ti.tmp2,ln_phi.tmp2,ln_sigma2.tmp2,b0.tmp2[zero_mask==1],b1.tmp2[one_mask==1]),propos.me[-(1:N)][1:n_par],propos.cov[-(1:N),-(1:N)][1:(n_par),1:(n_par)],log=T)
  ou5=mvtnorm::dmvnorm(tau.tmp2,propos.me[1:N],propos.cov[(1:N),(1:N)],log=T)

  log.l2[i]=sum(ou1)+sum(ou4)+ou2 - (ou5+ou3)
  debug1[i,1:5]=c(sum(ou1),sum((ou4)),ou2, (ou5), (ou3))
}

log.l1=mll1=c()
llb=matrix(,N,n1)
if(!silent) cat("\nStage 2/2:\n")
for(i in 1:n1){
  if(!silent) cat("processing posterior sample",i,"\r")
  tau.tmp1=post.batch2[i,tau.ind]
  ti.tmp1=post.batch2[i,ti.ind]
  ln_phi.tmp1=post.batch2[i,ln_phi.ind]
  ln_sigma2.tmp1=post.batch2[i,ln_sigma2.ind]
  if(inflated=="zero"|inflated=="both") b0.tmp1=post.batch2[i,b0.ind]%*%zero_mask_matrix else b0.tmp1=NULL
  if(inflated=="one"|inflated=="both") b1.tmp1=post.batch2[i,b1.ind]%*%one_mask_matrix else b1.tmp1=NULL

  b0.tmp1[zero_mask==0]=-Inf
  b1.tmp1[one_mask==0]=Inf

  ou5=ou1=ou2=ou3=ou4=c()
   for(p in 1:N){
      ind=which(z[p,]<1 & z[p,]>0 & z[p,]!=-999)
      tmp=c()
      if(model=="beta")
      tmp[ind] = stats::dbeta(z[p,ind],exp(.5*(exp(ln_phi.tmp1[ind])*tau.tmp1[p]+ti.tmp1[ind]+ln_sigma2.tmp1[ind])),exp(.5*(-exp(ln_phi.tmp1[ind])*tau.tmp1[p]-ti.tmp1[ind]+ln_sigma2.tmp1[ind])),log=T)
      if(model=="simplex")
      tmp[ind] = log(dsimplex(z[p,ind],m=1/(1+exp(-(ti.tmp1[ind]+exp(ln_phi.tmp1[ind])*tau.tmp1[p]))),s=exp(ln_sigma2.tmp1[ind])))
      if(model=="samejima")
      tmp[ind] = -log(z[p,ind])-log(1-z[p,ind])+stats::dnorm(stats::qlogis(z[p,ind]),ti.tmp1[ind]+exp(ln_phi.tmp1[ind])*tau.tmp1[p],exp(.5*ln_sigma2.tmp1[ind]),log=T)
      if(model=="normal")
      tmp[ind] = stats::dnorm(z[p,ind],ti.tmp1[ind]+exp(ln_phi.tmp1[ind])*tau.tmp1[p],exp(.5*ln_sigma2.tmp1[ind]),log=T)

      tmp[ind]=tmp[ind]+ log(  stats::plogis(b1.tmp1[ind]-exp(ln_phi.tmp1[ind])*tau.tmp1[p]) -  stats::plogis(b0.tmp1[ind]-exp(ln_phi.tmp1[ind])*tau.tmp1[p]))
      if(inflated=="zero"|inflated=="both"){
        ind=which(z[p,]==0)
        tmp[ind] =log( stats::plogis(b0.tmp1[ind]-exp(ln_phi.tmp1[ind])*tau.tmp1[p]))
      }
      if(inflated=="one"|inflated=="both"){
        ind=which(z[p,]==1)
        tmp[ind] =log( 1-stats::plogis(b1.tmp1[ind]-exp(ln_phi.tmp1[ind])*tau.tmp1[p]))
      }
      ou1[p]=sum(tmp[is.finite(tmp)],na.rm=T)
      ou4[p]=stats::dnorm(tau.tmp1[p],log=T)
      llb[p,i]=ou1[p]
    }
    ou2=sum(log(stats::dnorm(ln_phi.tmp1,0,10)))+sum(log(stats::dnorm(ln_sigma2.tmp1,0,10))+log(stats::dnorm(ti.tmp1,0,10))) +sum(log(stats::dnorm(b0.tmp1[zero_mask==1],0,10)))+sum(log(stats::dnorm(b1.tmp1[one_mask==1],0,10)/stats::pnorm(b0.tmp1[one_mask==1],0,10,low=F)))
    ou3=mvtnorm::dmvnorm(c(ti.tmp1,ln_phi.tmp1,ln_sigma2.tmp1,b0.tmp1[zero_mask==1],b1.tmp1[one_mask==1]),propos.me[-(1:N)][1:(n_par)],propos.cov[-(1:N),-(1:N)][1:n_par,1:n_par],log=T)
    ou5=mvtnorm::dmvnorm(tau.tmp1,propos.me[1:N],propos.cov[(1:N),(1:N)],log=T)

  log.l1[i]=sum(ou1) + sum(ou4)+ou2 - (ou5+ou3)
  mll1[i]= sum(ou1) + sum(ou4)
}

r0=.2
r1=c()
r1[1]=0.1
l_star1=stats::median(log.l1)
it=1
while(abs(log(rev(r1)[1])-log(r0))>tol & it<niter){
  r1[it+1]= mean(exp(log.l2-l_star1) / (.5 * exp(log.l2-l_star1) + .5 * r1[it])) /  mean(1 / (.5 * exp(log.l1-l_star1) + .5 * r1[it]))
  r0=r1[it]
  it=it+1
}
if(it==niter) stop("Iterative scheme did not converge, increase niter or tol")

# This code can be used to calculate the conditional DIC and WAIC. However, it is generally not recommended to use these for latent variable
# models like those in boundIRT. See ?loglikBIRT for alternatives (marginal WAIC and LOOIC)
#
# ###########
# #DEVIANCE #
# ###########
# if(!silent) cat("\nStage 3/3 (DIC and WAIC):\n")
#
# lla=matrix(,N,n1)
# for(i in 1:n1){
#   if(!silent) cat("processing sample: ",i,"\r")
#   tau.tmp1=post.batch1[i,tau.ind]
#   ti.tmp1=post.batch1[i,ti.ind]
#   ln_phi.tmp1=post.batch1[i,ln_phi.ind]
#   ln_sigma2.tmp1=post.batch1[i,ln_sigma2.ind]
#   if(inflated=="zero"|inflated=="both") b0.tmp1=post.batch1[i,b0.ind]%*%zero_mask_matrix else b0.tmp1=NULL
#   if(inflated=="one"|inflated=="both") b1.tmp1=post.batch1[i,b1.ind]%*%one_mask_matrix else b1.tmp1=NULL
#
#   b0.tmp1[zero_mask==0]=-Inf
#   b1.tmp1[one_mask==0]=Inf
#
#   ou5=ou1=ou2=ou3=ou4=c()
#   for(p in 1:N){
#      ind=which(z[p,]<1 & z[p,]>0 & z[p,]!=-999)
#      tmp=c()
#      if(model=="beta")
#      tmp[ind] = stats::dbeta(z[p,ind],exp(.5*(exp(ln_phi.tmp1[ind])*tau.tmp1[p]+ti.tmp1[ind]+ln_sigma2.tmp1[ind])),exp(.5*(-exp(ln_phi.tmp1[ind])*tau.tmp1[p]-ti.tmp1[ind]+ln_sigma2.tmp1[ind])),log=T)
#      if(model=="simplex")
#      tmp[ind] = log(dsimplex(z[p,ind],m=1/(1+exp(-(ti.tmp1[ind]+exp(ln_phi.tmp1[ind])*tau.tmp1[p]))),s=exp(ln_sigma2.tmp1[ind])))
#      if(model=="samejima")
#      tmp[ind] = -log(z[p,ind])-log(1-z[p,ind])+stats::dnorm(stats::qlogis(z[p,ind]),ti.tmp1[ind]+exp(ln_phi.tmp1[ind])*tau.tmp1[p],exp(.5*ln_sigma2.tmp1[ind]),log=T)
#      if(model=="normal")
#      tmp[ind] = stats::dnorm(z[p,ind],ti.tmp1[ind]+exp(ln_phi.tmp1[ind])*tau.tmp1[p],exp(.5*ln_sigma2.tmp1[ind]),log=T)
#
#      tmp[ind]=tmp[ind]+ log(  stats::plogis(b1.tmp1[ind]-exp(ln_phi.tmp1[ind])*tau.tmp1[p]) -  stats::plogis(b0.tmp1[ind]-exp(ln_phi.tmp1[ind])*tau.tmp1[p]))
#      if(inflated=="zero"|inflated=="both"){
#        ind=which(z[p,]==0)
#        tmp[ind] =log( stats::plogis(b0.tmp1[ind]-exp(ln_phi.tmp1[ind])*tau.tmp1[p]))
#      }
#      if(inflated=="one"|inflated=="both"){
#        ind=which(z[p,]==1)
#        tmp[ind] =log( 1-stats::plogis(b1.tmp1[ind]-exp(ln_phi.tmp1[ind])*tau.tmp1[p]))
#      }
#      ou1[p]=sum(tmp[is.finite(tmp)],na.rm=T)
#      lla[p,i]=ou1[p]
#    }
# }
# dev.mean=c()
#   for(p in 1:N){
#      ind=which(z[p,]<1 & z[p,]>0 & z[p,]!=-999)
#      tmp=c()
#      if(model=="beta")
#      tmp[ind] = stats::dbeta(z[p,ind],exp(.5*(exp(ln_phi.est[ind])*tau.est[p]+ti.est[ind]+ln_sigma2.est[ind])),exp(.5*(-exp(ln_phi.est[ind])*tau.est[p]-ti.est[ind]+ln_sigma2.est[ind])),log=T)
#      if(model=="simplex")
#      tmp[ind] = log(dsimplex(z[p,ind],m=1/(1+exp(-(ti.est[ind]+exp(ln_phi.est[ind])*tau.est[p]))),s=exp(ln_sigma2.est[ind])))
#      if(model=="samejima")
#      tmp[ind] = -log(z[p,ind])-log(1-z[p,ind])+stats::dnorm(stats::qlogis(z[p,ind]),ti.est[ind]+exp(ln_phi.est[ind])*tau.est[p],exp(.5*ln_sigma2.est[ind]),log=T)
#      if(model=="normal")
#      tmp[ind] = stats::dnorm(z[p,ind],ti.est[ind]+exp(ln_phi.est[ind])*tau.est[p],exp(.5*ln_sigma2.est[ind]),log=T)
#
#      tmp[ind]=tmp[ind]+ log(  stats::plogis(b1.est[ind]-exp(ln_phi.est[ind])*tau.est[p]) -  stats::plogis(b0.est[ind]-exp(ln_phi.est[ind])*tau.est[p]))
#      if(inflated=="zero"|inflated=="both"){
#       ind=which(z[p,]==0)
#       tmp[ind] =log( stats::plogis(b0.est[ind]-exp(ln_phi.est[ind])*tau.est[p]))
#      }
#      if(inflated=="one"|inflated=="both"){
#       ind=which(z[p,]==1)
#       tmp[ind] =log( 1-stats::plogis(b1.est[ind]-exp(ln_phi.est[ind])*tau.est[p]))
#      }
#
#      dev.mean[p]=sum(tmp[is.finite(tmp)],na.rm=T)
#  }
 if(!silent) cat("\n")
# ll=cbind(lla,llb)
# DIC=2*mean(-2*apply(ll,2,sum))-sum(-2*dev.mean)
# WAIC = -2*sum(log(apply(exp(ll),1,mean))) + 2 * sum(apply(ll,1,stats::var))
return(list(r1=rev(r1)[1],l_star=l_star1))
}

rsimpl=function (n = 1, m, s) {

  #from rmutil by Swihart & Lindsey (2022) but with minor adjustment
  p=stats::runif(n)
  if(m>.99) m=.99
  if(m<.01) m=.01
  if(s<.05) s=.05
  if(p>.999) p=.999
  if(p<(1-.999)) p=(1-.999)

  h <- function(y) .C("psimplex_c", as.double(y), as.double(m[i]),
                      as.double(s[i]), as.double(1), len = as.integer(1), eps = as.double(1e-06),
                      pts = as.integer(5), max = as.integer(16), err = integer(1),
                      res = double(1))$res - p[i]  # C function is adapted from R package rmutils, see the source code for copyright
  len <- max(length(p), length(m), length(s))
  if (length(p) != len) {
    if (length(p) == 1)
      p <- rep(p, len)
    else stop("length of p incorrect")
  }
  if (length(m) != len) {
    if (length(m) == 1)
      m <- rep(m, len)
    else stop("length of m incorrect")
  }
  if (length(s) != len) {
    if (length(s) == 1)
      s <- rep(s, len)
    else stop("length of s incorrect")
  }
  tmp <- vector(mode = "numeric", len)
  for (i in 1:len) {
    interval <- c(1e-06, 0.999999)
    h1=h(interval[1])
    h2=h(interval[2])
    while ((h1*h2 > 0)){
      interval <- c(interval[1]/10, 1 - interval[1]/10)
      h1=h(interval[1])
      h2=h(interval[2])
    }
    if((h1*h2 < 0)) tmp[i] <- stats::uniroot(h, interval)$root
    else{
      tmp[i]=NA
      warning("NA's produced is simulating simplex IRT responses - stats::uniroot() failed")
    }
  }
  return(tmp)
}

#' @export
#' @method plot BoundIRT
plot.BoundIRT=function(x,what=c("icc","iic","tic","density"),items=NULL,separate=FALSE,range=c(-3,3),by=.001,ymax=NULL,xlab=NULL,ylab=NULL,main=NULL,col=NULL,lwd_prob=2, ...){
  object=x
  what=match.arg(what)
  par=object$output
  inflated=object$inflated
  model=object$model
  nit=nrow(par)
  beta=par[,3]
  alpha=par[,1]
  phi=par[,5]

  if(inflated=="zero" |inflated=="both") gamma0=par[,7] else gamma0=rep(-Inf,nit)
  if(inflated=="one") gamma1=par[,7] else gamma1=rep(Inf,nit)
  if(inflated=="both") gamma1=par[,9]

  theta=seq(range[1],range[2],by=by)
  if(is.null(items)) items=1:nit
  if(!is.null(items)){
    if(length(items)>nit) stop("invalid items argument")
    if(any(items<=0)) stop("invalid items argument")
  }
  out=matrix(,length(theta),length(items))

  if(what=="icc"){
    if(is.null(ylab)) ylab="expected response"
    if(is.null(xlab)) xlab="latent trait"
    if(is.null(main)) main="Item Charateristic Curve"

    for(i in 1:length(items)) out[,i]=iccBIRT(theta,alpha[items[i]],beta[items[i]],phi[items[i]],gamma0[items[i]],gamma1[items[i]],model=model)
  }

  if(what=="iic" | what=="tic"){
    if(is.null(ylab)) ylab="information"
    if(is.null(xlab)) xlab="latent trait"
    if(what=="iic") if(is.null(main)) main="Item Information Curve"
    if(what=="tic") if(is.null(main)) main="Test Information Curve"

    for(i in 1:length(items)) out[,i]=iicBIRT(theta,alpha[items[i]],beta[items[i]],phi[items[i]],gamma0[items[i]],gamma1[items[i]],model=model)
    if(what=="tic") out=apply(out,1,sum)
  }
  if(what=="density"){
    if(is.null(ylab)) ylab="density/probability"
    if(is.null(xlab)) xlab="z"
    if(is.null(main)) main="Model implied density"

    z=seq(0,1,by=by)
    out=matrix(,length(z),length(items))

    for(i in 1:length(items)) out[,i]=dBIRT(z,alpha[items[i]],beta[items[i]],phi[items[i]],gamma0[items[i]],gamma1[items[i]],model=model)
  }
  x0=x=theta
  out0=out
  if(what=="density"){
    gg=length(z)
    x=z
    ind=-c(1,gg)
    x0=x[ind]
    out0=out[ind,]
  }
  if(length(items)==1 | what=="tic") out0=matrix(out0)
  if(is.null(col)) col=rep(1:6,ceiling(length(items)/6))
  if(length(col)==1 & length(items!=1)) col=rep(col,length(items))

  if(is.null(ymax)) ymax=apply(out0,2,max)
  if(length(ymax)==1 & length(items!=1)) ymax=rep(ymax,length(items))

  if(!separate) graphics::matplot(x0,out0,type="l",lty=1,lwd=2,xlab=xlab,ylab=ylab,main=main,col=col,ylim=c(0,max(ymax)),...)
  for(i in 1:length(items)){
    if(separate) plot(x0,out0[,i],type="l",lty=1,lwd=2,xlab=xlab,ylab=ylab,main=main,col=col[i],ylim=c(0,ymax[i]),...)

    if(what=="density"){
        if(is.finite(gamma0[items[i]])) {
          graphics::segments(0,0,0,out[1,i],lwd=lwd_prob,col="grey")
          graphics::points(0,out[1,i],pch=16,col=col[i])
        }
        if(is.finite(gamma1[items[i]])){
          graphics::segments(1,0,1,out[gg,i],lwd=lwd_prob,col="grey")
          graphics::points(1,out[gg,i],pch=16,col=col[i])
        }
      }
  }

  invisible(list(x=x,out=out))
}

SB_mean=function(gamma, delta, n_terms = 50) {
  #Equation 56 from Johnson (1949)
  n = 1:n_terms
  
  mu=c()
  size=length(gamma)
  if(length(delta)==1) delta=rep(delta,size)
  
  for(i in 1:size){
    
    part1 = 0.5 / delta[i]
    
    arg_exp2  = -n^2 / (2 * delta[i]^2)
    arg_cosh  =  n * (1 - 2 * gamma[i] * delta[i]) / (2 * delta[i]^2)
    arg_sech  =  n / (2 * delta[i]^2)
    
    terms2 = exp(arg_exp2) * cosh(arg_cosh) / cosh(arg_sech)
    part2  = sum(terms2) / delta[i]
    
    
    m         = 2 * n - 1
    arg_exp3  = -0.5 * m^2 * pi^2 * delta[i]^2
    arg_sin3  =  m * pi * gamma[i] * delta[i]
    arg_csch3 =  m * pi^2 * delta[i]^2
    
    terms3 = exp(arg_exp3) * sin(arg_sin3) / sinh(arg_csch3)  # cosech(x) = 1/sinh(x)
    part3  = -2 * pi * delta[i] * sum(terms3)
    
    numerator = part1 + part2 + part3
    
    arg_exp_d = -2 * n^2 * pi^2 * delta[i]^2
    arg_cos_d =  2 * n * pi * gamma[i] * delta[i]
    
    denominator = 1 + 2 * sum(exp(arg_exp_d) * cos(arg_cos_d))
    
    mu[i] = (1 / sqrt(2 * pi)) * exp(-0.5 * gamma[i]^2) * numerator / denominator
  }
  return(mu)
}

#' @export
latregBIRT=function(formula,X,object, method=c("posterior","Croon"),nboot=1000){
  method=match.arg(method)
  pairwise=any(is.na(X))
  mcmc.object=object$mcmc
  if(!is.matrix(X)) stop("X needs to be a matrix with column names corresponding to the predictors in 'formula'")
  if((all.vars(formula[[1]])!="~") | (!inherits(formula, "formula"))) stop("invalid formula, provide a formula in the form: ~VAR1+VAR2+VAR3, where VAR1-VAR3 refer to variables in X")
  if(!all(all.vars(formula[[2]])%in%colnames(X))) stop("all predictors from 'formula' should have corresponding column names in matrix X")

  theta_post=rstan::extract(mcmc.object,"theta")$theta
  N=ncol(theta_post)
  npar=length(all.vars(formula[[2]]))
  X=X[,all.vars(formula[[2]]),drop=FALSE]

  if(method=="posterior"){
    EAP=colMeans(theta_post)
    sds=apply(theta_post,2,stats::sd)
    r=stats::var(EAP)/(stats::var(EAP)+(sds^2))
    f=(stats::var(EAP)+(sds^2))
    covX=stats::cov(X,use="p")
    b=matrix(,nrow(theta_post),npar*2)

    for(jj in 1:nrow(theta_post)){
      y=theta_post[jj,]
      all=cbind(y,X)

      mu=matrix(colMeans(all),N,npar+1,TRUE)
      if(pairwise) corcov=cov_pairwise(all,r) else
        corcov=1/(N-1)*t(all-mu)%*%diag(1/r)%*%((all-mu))
      diag(corcov)[1]=1/(N-1)*sum((y-mean(y))^2/r^2-f*(1-r)/r)

      covXY=corcov[-1,1]
      ests=solve(covX)%*%covXY
      df=N - npar

      sig2= 1/df*(corcov[1,1]-t(ests)%*%covX%*%ests)
      samp_var=(df * sig2[1])/stats::rchisq(1, df)
      avar=samp_var*solve(covX)

      b[jj,1:npar]=MASS::mvrnorm(1,ests,avar)
      b[jj,(npar+1):(2*npar)]=sqrt(diag(avar))
    }
    output0=t(matrix(colMeans(b[,1:npar,drop=FALSE])))
    output0=rbind(output0,apply(b[,1:npar,drop=FALSE],2,stats::sd))
    rownames(output0)=c("mean","sd")
    output0=rbind(output0,apply(b[,1:npar,drop=FALSE],2,stats::quantile,prob=c(.025,.25,.50,.75,.975)))
    colnames(output0)=colnames(X)
    ests=rbind(NA,b)
    colnames(ests)=c(colnames(X),paste0("sd_",colnames(X)))
  }
  if(method=="Croon"){
    ests=matrix(,nboot+1,npar)
    EAPm=colMeans(theta_post)
    EAPs=apply(theta_post,2,stats::sd)
    for(ii in 0:nboot){
      if(ii>0) select=sample(1:N,replace=TRUE) else select=1:N

      y=EAPm[select]
      sds=EAPs[select]

      all=cbind(y,X[select,,drop=FALSE])

      r=stats::var(y)/(stats::var(y)+(sds^2))
      f=(stats::var(y)+(sds^2))

      mu=matrix(colMeans(all),N,npar+1,TRUE)
      if(pairwise) corcov=cov_pairwise(all,r) else
        corcov=1/(N-1)*t(all-mu)%*%diag(1/r)%*%((all-mu))
      diag(corcov)[1]=1/(N-1)*sum((y-mean(y))^2/r^2-f*(1-r)/r)

      covX=stats::cov(all[,-1,drop=FALSE],use="p")
      covXY=corcov[-1,1]
      b=solve(covX)%*%covXY
      ests[ii+1,]=t(b)
    }
    se_ests=apply(ests[-1,,drop=FALSE],2,stats::sd)
    output0=rbind(ests[1,,drop=FALSE],se_ests)
    rownames(output0)=c("est","se")
    colnames(ests)=colnames(X)
    colnames(output0)=colnames(X)
    }

  results=list(output=output0,samples=ests[-1,])
  class(results)="latregBIRT"
  return(results)
}

cov_pairwise=function(X,r) {
  p=ncol(X)
  S=matrix(NA, p, p)
  dimnames(S)[[1]]=colnames(X)
  dimnames(S)[[2]]=colnames(X)
  for (i in 1:p) {
    for (j in 1:p) {
      mask <- stats::complete.cases(X[, c(i, j)])
      xi=X[mask, i]
      xj=X[mask, j]
      n=sum(mask)
      S[i, j]=sum((xi-mean(xi))*(1/r[mask])*(xj-mean(xj)))/(n-1)
    }
  }

  return(S)
}

bridgeBIRTno=function(object, niter,tol,silent){

fit=object$mcmc
z=object$z
model=object$model
z=as.matrix(z)
item.out=object$output
inflated=object$inflated

tau.samples=rstan::extract(fit,"theta")$theta
ti.samples=rstan::extract(fit,"beta")$beta
ln_phi.samples=log(rstan::extract(fit,"alpha")$alpha)
ln_sigma2.samples=log(rstan::extract(fit,"phi")$phi)

tau.est=apply(tau.samples,2,mean)
ti.est=apply(ti.samples,2,mean)
ln_phi.est=apply(ln_phi.samples,2,mean)
ln_sigma2.est=apply(ln_sigma2.samples,2,mean)

N=length(tau.est)
nit=length(ti.est)
no_iter=nrow(tau.samples)

###
n1=n2=floor(no_iter/2)

full.samples=cbind(tau.samples,ti.samples,ln_phi.samples,ln_sigma2.samples)
tau.ind=1:N
ti.ind=1:nit+N
ln_phi.ind=1:nit+N+nit
ln_sigma2.ind=1:nit+N+nit*2

post.batch1=full.samples[1:n1,]
post.batch2=full.samples[(n1+1):(2*n1),]

propos.me.p=apply(post.batch1[,tau.ind],2,mean)
propos.me.i=apply(post.batch1[,(N+1):(N+nit*3)],2,mean)

propos.cov.p=diag(diag(stats::cov(post.batch1[,tau.ind])))
propos.cov.i=stats::cov(post.batch1[,(N+1):(N+nit*3)])

propos.samples.p=MASS::mvrnorm(n2,propos.me.p,propos.cov.p)
propos.samples.i=MASS::mvrnorm(n2,propos.me.i,propos.cov.i)

propos.samples=cbind(propos.samples.p,propos.samples.i)

propos.me=c(propos.me.p,propos.me.i)
propos.cov=rbind(cbind(propos.cov.p,propos.cov.p[,1:(3*nit)]*0),cbind(propos.cov.p[1:(3*nit),]*0,propos.cov.i))

debug1=matrix(,n1,5)

dsimplex=function (y, m, s2){  #from rmutil by Swihart & Lindsey (2022) but with minor adjustment
  tmp <- -((y - m)/(m * (1 - m)))^2/(2 * y * (1 - y) * s2) -
    (log(2 * pi * s2) + 3 * (log(y) + log(1 - y)))/2
  return(exp(tmp))
}


log.l2=c()
if(!silent) cat("Stage 1/2:\n")
for(i in 1:n2){
  if(!silent) cat("processing proposal sample",i,"\r")
  tau.tmp2=propos.samples[i,tau.ind]
  ti.tmp2=propos.samples[i,ti.ind]
  ln_phi.tmp2=propos.samples[i,ln_phi.ind]
  ln_sigma2.tmp2=propos.samples[i,ln_sigma2.ind]

  ou5=ou1=ou2=ou3=ou4=c()
  for(p in 1:N){
    if(model=="beta") ou1[p]=sum(stats::dbeta(z[p,],exp(.5*(exp(ln_phi.tmp2)*tau.tmp2[p]+ti.tmp2+ln_sigma2.tmp2)),exp(.5*(-exp(ln_phi.tmp2)*tau.tmp2[p]-ti.tmp2+ln_sigma2.tmp2)),log=T))
    if(model=="simplex"){
      tmp=log(dsimplex(z[p,],m=1/(1+exp(-(ti.tmp2+exp(ln_phi.tmp2)*tau.tmp2[p]))),s=exp(ln_sigma2.tmp2)))
      ou1[p]=sum(tmp[is.finite(tmp)])
    }
    if(model=="samejima") ou1[p]=sum(-log(z[p,])-log(1-z[p,])+stats::dnorm(stats::qlogis(z[p,]),ti.tmp2+exp(ln_phi.tmp2)*tau.tmp2[p],exp(.5*ln_sigma2.tmp2),log=T))
    if(model=="normal") ou1[p]=sum(stats::dnorm(z[p,],ti.tmp2+exp(ln_phi.tmp2)*tau.tmp2[p],exp(.5*ln_sigma2.tmp2),log=T))

    ou4[p]=stats::dnorm(tau.tmp2[p],log=T)
  }
  ou2=sum(log(stats::dnorm(ln_phi.tmp2,0,10))+log(stats::dnorm(ln_sigma2.tmp2,0,10))+log(stats::dnorm(ti.tmp2,0,10)))
  ou3=mvtnorm::dmvnorm(c(ti.tmp2,ln_phi.tmp2,ln_sigma2.tmp2),propos.me[-(1:N)][1:(3*nit)],propos.cov[-(1:N),-(1:N)][1:(3*nit),1:(3*nit)],log=T)
  ou5=mvtnorm::dmvnorm(tau.tmp2,propos.me[1:N],propos.cov[(1:N),(1:N)],log=T)

  log.l2[i]=sum(ou1)+sum(ou4)+ou2 - (ou5+ou3)
  debug1[i,1:5]=c(sum(ou1),sum((ou4)),ou2, (ou5), (ou3))
}

log.l1=mll1=c()
llb=matrix(,N,n1)
if(!silent) cat("\nStage 2/2:\n")
for(i in 1:n1){
  if(!silent) cat("processing posterior sample",i,"\r")
  tau.tmp1=post.batch2[i,tau.ind]
  ti.tmp1=post.batch2[i,ti.ind]
  ln_phi.tmp1=post.batch2[i,ln_phi.ind]
  ln_sigma2.tmp1=post.batch2[i,ln_sigma2.ind]

  ou5=ou1=ou2=ou3=ou4=c()
  for(p in 1:N){
    if(model=="beta") ou1[p]=sum(stats::dbeta(z[p,],exp(.5*(exp(ln_phi.tmp1)*tau.tmp1[p]+ti.tmp1+ln_sigma2.tmp1)),exp(.5*(-exp(ln_phi.tmp1)*tau.tmp1[p]-ti.tmp1+ln_sigma2.tmp1)),log=T))
    if(model=="simplex"){
      tmp=log(dsimplex(z[p,],m=1/(1+exp(-(ti.tmp1+exp(ln_phi.tmp1)*tau.tmp1[p]))),s=exp(ln_sigma2.tmp1)))
      ou1[p]=ou1[p]=sum(tmp[is.finite(tmp)])
    }
    if(model=="samejima") ou1[p]=sum(-log(z[p,])-log(1-z[p,])+stats::dnorm(stats::qlogis(z[p,]),ti.tmp1+exp(ln_phi.tmp1)*tau.tmp1[p],exp(.5*ln_sigma2.tmp1),log=T))
    if(model=="normal") ou1[p]=sum(stats::dnorm(z[p,],ti.tmp1+exp(ln_phi.tmp1)*tau.tmp1[p],exp(.5*ln_sigma2.tmp1),log=T))

    ou4[p]=stats::dnorm(tau.tmp1[p],log=T)
    llb[p,i]= ou1[p]
  }
  ou2=sum(log(stats::dnorm(ln_phi.tmp1,0,10))+log(stats::dnorm(ln_sigma2.tmp1,0,10))+log(stats::dnorm(ti.tmp1,0,10)))
  ou3=mvtnorm::dmvnorm(c(ti.tmp1,ln_phi.tmp1,ln_sigma2.tmp1),propos.me[-(1:N)][1:(3*nit)],propos.cov[-(1:N),-(1:N)][1:(3*nit),1:(3*nit)],log=T)
  ou5=mvtnorm::dmvnorm(tau.tmp1,propos.me[1:N],propos.cov[(1:N),(1:N)],log=T)
  log.l1[i]=sum(ou1) + sum(ou4)+ou2 - (ou5+ou3)
  mll1[i]= sum(ou1) + sum(ou4)
}

r0=.2
r1=c()
r1[1]=0.1
l_star1=stats::median(log.l1)
it=1
while(abs(log(rev(r1)[1])-log(r0))>tol & it<niter){
  r1[it+1]= mean(exp(log.l2-l_star1) / (.5 * exp(log.l2-l_star1) + .5 * r1[it])) /  mean(1 / (.5 * exp(log.l1-l_star1) + .5 * r1[it]))
  r0=r1[it]
  it=it+1
}
if(it==niter) stop("Iterative scheme did not converge, increase niter or tol")

# This code can be used to calculate the conditional DIC and WAIC. However, it is generally not recommended to use these for latent variable
# models like those in boundIRT. See ?loglikBIRT for alternatives (marginal WAIC and LOOIC)
#
# ###########
# #DEVIANCE #
# ###########
# if(!silent) cat("\nStage 3/3 (DIC and WAIC):\n")
#
# lla=matrix(,N,n1)
# for(i in 1:n1){
#   if(!silent) cat("processing sample: ",i,"\r")
#   tau.tmp1=post.batch1[i,tau.ind]
#   ti.tmp1=post.batch1[i,ti.ind]
#   ln_phi.tmp1=post.batch1[i,ln_phi.ind]
#   ln_sigma2.tmp1=post.batch1[i,ln_sigma2.ind]
#
#   ou5=ou1=ou2=ou3=ou4=c()
#   for(p in 1:N){
#     if(model=="beta") ou1[p]=sum(stats::dbeta(z[p,],exp(.5*(exp(ln_phi.tmp1)*tau.tmp1[p]+ti.tmp1+ln_sigma2.tmp1)),exp(.5*(-exp(ln_phi.tmp1)*tau.tmp1[p]-ti.tmp1+ln_sigma2.tmp1)),log=T))
#     if(model=="simplex"){
#       tmp=log(dsimplex(z[p,],m=1/(1+exp(-(ti.tmp1+exp(ln_phi.tmp1)*tau.tmp1[p]))),s=exp(ln_sigma2.tmp1)))
#       ou1[p]=sum(log(dsimplex(z[p,],m=1/(1+exp(-(ti.tmp1+exp(ln_phi.tmp1)*tau.tmp1[p]))),s=exp(ln_sigma2.tmp1))))
#     }
#     if(model=="samejima") ou1[p]=sum(-log(z[p,])-log(1-z[p,])+stats::dnorm(stats::qlogis(z[p,]),ti.tmp1+exp(ln_phi.tmp1)*tau.tmp1[p],exp(.5*ln_sigma2.tmp1),log=T))
#     if(model=="normal") ou1[p]=sum(stats::dnorm(z[p,],ti.tmp1+exp(ln_phi.tmp1)*tau.tmp1[p],exp(.5*ln_sigma2.tmp1),log=T))
#
#     lla[p,i]= ou1[p]
#   }
# }
# dev.mean=c()
# for(p in 1:N){
#   if(model=="beta") dev.mean[p]=sum(stats::dbeta(z[p,],exp(.5*(exp(ln_phi.est)*tau.est[p]+ti.est+ln_sigma2.est)),exp(.5*(-exp(ln_phi.est)*tau.est[p]-ti.est+ln_sigma2.est)),log=T))
#   if(model=="simplex"){
#     tmp=log(dsimplex(z[p,],m=1/(1+exp(-(ti.est+exp(ln_phi.est)*tau.est[p]))),s=exp(ln_sigma2.est)))
#     dev.mean[p]=sum(tmp[is.finite(tmp)])
#   }
#   if(model=="samejima") dev.mean[p]=sum(-log(z[p,])-log(1-z[p,])+stats::dnorm(stats::qlogis(z[p,]),ti.est+exp(ln_phi.est)*tau.est[p],exp(.5*ln_sigma2.est),log=T))
#   if(model=="normal") dev.mean[p]=sum(stats::dnorm(z[p,],ti.est+exp(ln_phi.est)*tau.est[p],exp(.5*ln_sigma2.est),log=T))
# }
#
 if(!silent) cat("\n")
# ll=cbind(lla,llb)
# DIC=2*mean(-2*apply(ll,2,sum))-sum(-2*dev.mean)
# WAIC = -2*sum(log(apply(exp(ll),1,mean))) + 2 * sum(apply(ll,1,stats::var))
return(list(r1=rev(r1)[1],l_star=l_star1))
}

iccBIRT=function(theta,alpha,beta,phi,gamma0=NULL,gamma1=NULL,model=c("beta","samejima","simplex","normal")){
  model=match.arg(model)
  if(is.null(gamma0)) gamma0=-Inf
  if(is.null(gamma1)) gamma1=Inf

  if((length(beta)!=1) |   (length(alpha)!=1) | (length(phi)!=1) | (length(gamma0)!=1) | (length(gamma1)!=1))
    stop("Item parameters should be of length 1")


  if(model%in%c("beta","simplex"))
    out=(stats::plogis(gamma1-alpha*theta) - stats::plogis(gamma0-alpha*theta))*stats::plogis(alpha*theta+beta)+(1-stats::plogis(gamma1-alpha*theta))

  if(model=="samejima")
    out=(stats::plogis(gamma1-alpha*theta) - stats::plogis(gamma0-alpha*theta))*
        SB_mean(-(alpha*theta+beta)/sqrt(phi),1/sqrt(phi))+
        (1-stats::plogis(gamma1-alpha*theta))

  if(model=="normal")
    out=(stats::plogis(gamma1-alpha*theta) - stats::plogis(gamma0-alpha*theta))*(alpha*theta+beta)+(1-stats::plogis(gamma1-alpha*theta))

  return(out)
}

dBIRT=function(z,alpha,beta,phi,gamma0=NULL,gamma1=NULL,model=c("beta","samejima","simplex","normal")){
    model=match.arg(model)
    if(phi<0) stop("The dispersion parameter phi should be larger than 0")
    if(is.null(gamma0)) gamma0=-Inf
    if(is.null(gamma1)) gamma1=Inf
    if(any(z>1) | any(z<0)) stop("Item scores shoud be between 0 and 1")

 if(model=="beta"){
    marg=function(theta,z,beta,alpha,gamma0,gamma1,phi){
    ln_phi=log(phi)
    if(z!=0 & z!=1 & z!=-999){
       a = exp(.5* (alpha*theta + beta+ln_phi));
       b  = exp(.5* (-alpha*theta - beta+ln_phi));
       out= (stats::plogis(gamma1-alpha*theta) - stats::plogis(gamma0-alpha*theta))*stats::dbeta(z,a,b)
    }
    else if(z==0) out=stats::plogis(gamma0-alpha*theta)
    else if(z==1) out=1-stats::plogis(gamma1-alpha*theta)
  return(out*stats::dnorm(theta))
  }
  }

 if(model=="simplex"){
   dsimplex=function (y, m, s2){  #from rmutil by Swihart & Lindsey (2022) but with minor adjustment
     tmp <- -((y - m)/(m * (1 - m)))^2/(2 * y * (1 - y) * s2) -
       (log(2 * pi * s2) + 3 * (log(y) + log(1 - y)))/2
     return(exp(tmp))
   }
  marg=function(theta,z,beta,alpha,gamma0,gamma1,phi){
    if(z!=0 & z!=1 & z!=-999) out= (stats::plogis(gamma1-alpha*theta) - stats::plogis(gamma0-alpha*theta))*dsimplex(z,stats::plogis(beta+alpha*theta),phi)
    else if(z==0) out=stats::plogis(gamma0-alpha*theta)
    else if(z==1) out=1-stats::plogis(gamma1-alpha*theta)
  return(out*stats::dnorm(theta))
  }
  }

 if(model=="samejima"){
 marg=function(theta,z,beta,alpha,gamma0,gamma1,phi){
    if(z!=0 & z!=1 & z!=-999) out= (stats::plogis(gamma1-alpha*theta) - stats::plogis(gamma0-alpha*theta))*1/(z*(1-z))*stats::dnorm(stats::qlogis(z),beta+alpha*theta,sqrt(phi))
    else if(z==0) out=stats::plogis(gamma0-alpha*theta)
    else if(z==1) out=1-stats::plogis(gamma1-alpha*theta)
  return(out*stats::dnorm(theta))
  }
 }
 if(model=="normal"){
  marg=function(theta,z,beta,alpha,gamma0,gamma1,phi){
    if(z!=0 & z!=1 & z!=-999) out= (stats::plogis(gamma1-alpha*theta) - stats::plogis(gamma0-alpha*theta))*stats::dnorm(z,beta+alpha*theta,sqrt(phi))
    else if(z==0) out=stats::plogis(gamma0-alpha*theta)
    else if(z==1) out=1-stats::plogis(gamma1-alpha*theta)
    return(out*stats::dnorm(theta))
  }
}

  ou=c()
  for(i in 1:length(z)){
    ou[i]=stats::integrate(marg,-Inf,Inf,z=z[i],beta=beta,alpha=alpha,gamma0=gamma0,gamma1=gamma1,phi=phi)$value
  }
return(ou)
}

#' @export

loglikBIRT=function(object,marginal=TRUE,nquad=15,silent=FALSE){
  mcmc=object$mcmc
  model=object$model
  inflated=object$inflated
  N=nrow(object$z)
  nit=ncol(object$z)
  item.out=object$output
  z=as.matrix(object$z)
  z[z==-999]=NA
  alpha=rstan::extract(mcmc,"alpha")$"alpha"
  nsamples0=nrow(alpha)
  nsamples=nsamples0
  index=rev(seq(nsamples0,nsamples0-nsamples,by=-1))
  alpha=alpha[index,]
  beta=rstan::extract(mcmc,"beta")$"beta"[index,]
  phi=rstan::extract(mcmc,"phi")$"phi"[index,]
  if(model=="beta") ln_phi=log(phi)
  if(model=="simplex"){
    dsimplex=function (y, m, s2){  #from rmutil by Swihart & Lindsey (2022) but with minor adjustment
      tmp <- -((y - m)/(m * (1 - m)))^2/(2 * y * (1 - y) * s2) -
        (log(2 * pi * s2) + 3 * (log(y) + log(1 - y)))/2
      return(exp(tmp))
    }
  }
  theta=rstan::extract(mcmc,"theta")$"theta"[index,]


  if(inflated=="zero" | inflated=="both") gamma0_c=rstan::extract(mcmc,"gamma0")$"gamma0"[index,] else gamma0_c=matrix(-Inf,nsamples,nit)
  if(inflated=="one" | inflated=="both") gamma1_c=rstan::extract(mcmc,"gamma1")$"gamma1"[index,] else gamma1_c=matrix(Inf,nsamples,nit)


  if(inflated=="zero" | inflated=="both"){
    if(inflated=="zero"){
      one_mask=rep(0,nit)
      one_mask_matrix=diag(0,nit)
    }
    zero_mask=1*(is.finite(item.out[,"gamma0"]))
    zero_mask_matrix=matrix(0,sum(zero_mask),nit)
    for(i in 1:nit){
      if(zero_mask[i]==1) zero_mask_matrix[sum(zero_mask[1:i]),i]=1
      if(zero_mask[i]==0) zero_mask_matrix[sum(zero_mask[1:i]),i]=0
    }
  }

  if(inflated=="one" | inflated=="both"){
    if(inflated=="one"){
      zero_mask=rep(0,nit)
      zero_mask_matrix=diag(0,nit)
    }
    one_mask=1*(is.finite(item.out[,"gamma1"]))
    one_mask_matrix=matrix(0,sum(one_mask),nit)
    for(i in 1:nit){
      if(one_mask[i]==1) one_mask_matrix[sum(one_mask[1:i]),i]=1
      if(one_mask[i]==0) one_mask_matrix[sum(one_mask[1:i]),i]=0
    }
  }

  gamma0=gamma0_c%*%zero_mask_matrix
  gamma0[,zero_mask==0]=-Inf
  gamma1=gamma1_c%*%one_mask_matrix
  gamma1[,one_mask==0]=Inf

  if(!marginal){
    log_lik=array(,dim = c(nsamples,N,nit))
    a=b=rep(0,nit)
    out=rep(NA,nit)
    for(s in 1:nsamples){
      if(!silent) cat("Calculating conditional log_lik for posterior sample: ",s,"\r")
      for(p in 1:N){
          ind=(z[p,]!=0) & (z[p,]!=1) & (!is.na(z[p,]))
          ind0=(z[p,]==0) & (!is.na(z[p,]))
          ind1=(z[p,]==1) & (!is.na(z[p,]))
          prob01=stats::plogis(gamma1[s,ind]-alpha[s,ind]*theta[s,p]) - stats::plogis(gamma0[s,ind]-alpha[s,ind]*theta[s,p])

          if(model=="beta"){
              a[ind] = exp(.5* (alpha[s,ind]*theta[s,p] + beta[s,ind]+ln_phi[s,ind]));
              b[ind]  = exp(.5* (-alpha[s,ind]*theta[s,p] - beta[s,ind]+ln_phi[s,ind]));
              out[ind]= prob01*stats::dbeta(z[p,ind],a[ind],b[ind])
          }
          if(model=="simplex")
            out[ind] = prob01*dsimplex(z[p,ind],stats::plogis(beta[s,ind]+alpha[s,ind]*theta[s,p]),phi[s,ind])

          if(model=="samejima")
            out[ind]= prob01*1/(z[p,ind]*(1-z[p,ind]))*stats::dnorm(stats::qlogis(z[p,ind]),beta[s,ind]+alpha[s,ind]*theta[s,p],sqrt(phi[s,ind]))

          if(model=="normal")
            out[ind]= prob01*stats::dnorm(z[p,ind],beta[s,ind]+alpha[s,ind]*theta[s,p],sqrt(phi[s,ind]))

          out[ind0]=stats::plogis(gamma0[s,ind0]-alpha[s,ind0]*theta[s,p])
          out[ind1]=1-stats::plogis(gamma1[s,ind1]-alpha[s,ind1]*theta[s,p])
          log_lik[s,p,]=log(out)

      }
    }
  log_lik=apply(log_lik,c(1,2),sum)
  }
  if(marginal){
    Quad=statmod::gauss.quad(nquad,kind="hermite")
    W=Quad$weights/sqrt(pi)
    A=Quad$nodes*sqrt(2)

    log_lik=matrix(,nsamples,N)
    a=b=rep(0,nit)
    out=rep(NA,nit)
    for(s in 1:nsamples){
      if(!silent) cat("Calculating marginal log_lik for posterior sample: ",s,"\r")
      for(p in 1:N){
        tmp=0
        for(k in 1:nquad){
          ind=(z[p,]!=0) & (z[p,]!=1) & (!is.na(z[p,]))
          ind0=(z[p,]==0) & (!is.na(z[p,]))
          ind1=(z[p,]==1) & (!is.na(z[p,]))
          prob01=stats::plogis(gamma1[s,ind]-alpha[s,ind]*A[k]) - stats::plogis(gamma0[s,ind]-alpha[s,ind]*A[k])

          if(model=="beta"){
            a[ind] = exp(.5* (alpha[s,ind]*A[k] + beta[s,ind]+ln_phi[s,ind]));
            b[ind]  = exp(.5* (-alpha[s,ind]*A[k] - beta[s,ind]+ln_phi[s,ind]));
            out[ind]= prob01*stats::dbeta(z[p,ind],a[ind],b[ind])
          }
          if(model=="simplex")
            out[ind] = prob01*dsimplex(z[p,ind],stats::plogis(beta[s,ind]+alpha[s,ind]*A[k]),phi[s,ind])

          if(model=="samejima")
            out[ind]= prob01*1/(z[p,ind]*(1-z[p,ind]))*stats::dnorm(stats::qlogis(z[p,ind]),beta[s,ind]+alpha[s,ind]*A[k],sqrt(phi[s,ind]))

          if(model=="normal")
            out[ind]= prob01*stats::dnorm(z[p,ind],beta[s,ind]+alpha[s,ind]*A[k],sqrt(phi[s,ind]))

          out[ind0]=stats::plogis(gamma0[s,ind0]-alpha[s,ind0]*A[k])
          out[ind1]=1-stats::plogis(gamma1[s,ind1]-alpha[s,ind1]*A[k])
          tmp=tmp+W[k]*prod(out,na.rm=TRUE)
        }
        log_lik[s,p]=log(tmp)
      }
    }
  }
  if(!silent) cat("\n")
  return(log_lik)
}

#' @export
rBIRT=function(N,nit,alpha,beta,phi,theta,gamma0=NULL,gamma1=NULL,model=c("beta","samejima","simplex","normal")){
  model=match.arg(model)
  if(length(alpha)!=nit | length(beta)!=nit | length(phi)!=nit) stop("Item parameter vectors need to be of length nit")
  if(length(theta)!=N) stop("Person parameter vector needs to be of length N")
  ln_phi=log(phi)
  if(is.null(gamma0)) gamma0=rep(-Inf,nit)
    else if(length(gamma0)!=nit) stop("Inflation parameter vectors needs to be of length nit")
  if(is.null(gamma1)) gamma1=rep(Inf,nit)
    else if(length(gamma1)!=nit) stop("Inflation parameter vectors needs to be of length nit")

  z=matrix(,N,nit)
  if(model=="beta"){
    for (p in 1:N)
      for (i in 1:nit){{
        #note that ln_phi corresponds to \tau_j in Equation 11-15 from Noel & Dauvier (2007)
        #in addition, phi in the present Package corresponds to phi^2 in Noel & Dauvier (2007) for better alignment with the other models in this package
        C=sample(0:2,1,prob=c(stats::plogis(gamma1[i]-alpha[i]*theta[p])-stats::plogis(gamma0[i]-alpha[i]*theta[p]),stats::plogis(gamma0[i]-alpha[i]*theta[p]),1-stats::plogis(gamma1[i]-alpha[i]*theta[p])))
        if(C==0) z[p,i]=stats::rbeta(1,exp(.5*(alpha[i]*theta[p]+beta[i]+ln_phi[i])),exp(.5*(-alpha[i]*theta[p]-beta[i]+ln_phi[i])))
        else z[p,i]=C-1
      }}
  }
  if(model=="samejima"){
    for (p in 1:N)
      for (i in 1:nit){{
        C=sample(0:2,1,prob=c(stats::plogis(gamma1[i]-alpha[i]*theta[p])-stats::plogis(gamma0[i]-alpha[i]*theta[p]),stats::plogis(gamma0[i]-alpha[i]*theta[p]),1-stats::plogis(gamma1[i]-alpha[i]*theta[p])))
        if(C==0) z[p,i]=stats::plogis(alpha[i]*theta[p]+beta[i]+stats::rnorm(1,0,sqrt(phi[i])))
        else z[p,i]=C-1
      }}
  }
  if(model=="simplex"){
    for (p in 1:N)
      for (i in 1:nit){{
        C=sample(0:2,1,prob=c(stats::plogis(gamma1[i]-alpha[i]*theta[p])-stats::plogis(gamma0[i]-alpha[i]*theta[p]),stats::plogis(gamma0[i]-alpha[i]*theta[p]),1-stats::plogis(gamma1[i]-alpha[i]*theta[p])))
        if(C==0) z[p,i]=rsimpl(1,m=1/(1+exp(-(beta[i]+alpha[i]*theta[p]))),s=phi[i])
        else z[p,i]=C-1
      }}
  }
  if(model=="normal"){
    for (p in 1:N)
      for (i in 1:nit){{
        C=sample(0:2,1,prob=c(stats::plogis(gamma1[i]-alpha[i]*theta[p])-stats::plogis(gamma0[i]-alpha[i]*theta[p]),stats::plogis(gamma0[i]-alpha[i]*theta[p]),1-stats::plogis(gamma1[i]-alpha[i]*theta[p])))
        if(C==0) z[p,i]=alpha[i]*theta[p]+beta[i]+stats::rnorm(1,0,sqrt(phi[i]))
        else z[p,i]=C-1
      }}
  }
return(z)
}

#' The 'BoundIRT' package.
#'
#' @description Bounded continuous data are encountered in many areas of test application.
#' Examples include visual analogue scales used in the measurement of personality, mood,
#' depression, and quality of life; item response times from tests with item deadlines;
#' confidence ratings; and pain intensity ratings. Using this package, item response theory (IRT)
#' models suitable for bounded continuous item scores can be fitted to data within a Bayesian framework.
#' The package draws on posterior sampling facilities provided by R-package 'rstan' (Stan Development Team, 2025).
#' Available models include the Beta IRT model by Noel and Dauvier (2007), the continuous response
#' model by Samejima (1973), the unbounded normal model by Mellenbergh (1994),
#' and the Simplex IRT model by Flores et al. (2020). All models can be
#' fitted with or without zero-one inflation (Molenaar et al., 2022).
#' Model fit comparisons can be conducted using the Watanabe–Akaike information criterion (WAIC),
#' the leave-one-out cross-validation information criterion (LOOIC), and the fully marginalized likelihood
#' (i.e., Bayes factors).
#'
#' @name BoundIRT-package
#' @aliases boundIRT
#' @useDynLib BoundIRT, .registration = TRUE
#' @import methods
#' @import Rcpp
#' @importFrom rstan sampling
#' @importFrom rstantools rstan_config
#' @importFrom RcppParallel RcppParallelLibs
#'
#' @references Flores, S., Bazan, J.L., & Bolfarine, H. (2020). A hierarchical
#'  joint model for bounded response time and response accuracy. In M. Wiberg,
#'  D. Molenaar, J. González,U. Bockenholt, & K. S. Kim (Eds.), \emph{Quantitative
#'  psychology: The 84th Annual Meeting of the Psychometric Society, Santiago de
#'  Chile, Chile} (pp. 95-109). Springer. \doi{10.1007/978-3-030-43469-4_8}
#'
#'   Mellenbergh, G. J. (1994). A unidimensional latent trait model for continuous
#'   item responses. \emph{Multivariate Behavioral Research}, \bold{29}(3), 223-236.
#'   \doi{10.1207/s15327906mbr2903_2}
#'
#'   Molenaar, D., Cúri, M., & Bazán, J.L. (2022). Zero and One Inflated Item Response
#'   Theory Models for Bounded Continuous Data. \emph{Journal of Educational and
#'   Behavioral Statistics}, \bold{47}, 693-735. \doi{10.3102/10769986221108455}
#'
#'   Noel, Y., & Dauvier, B. (2007). A beta item response model for continuous
#'   bounded responses. \emph{Applied Psychological Measurement}, \bold{31}(1),
#'   47-73. \doi{10.1177/0146621605287691}
#'
#'  Samejima, F. (1973). Homogeneous case of the continuous response model. \emph{Psychometrika},
#'  \bold{38}(2), 203-219.\doi{10.1007/BF02291114}
#'
#'   Stan Development Team (2025). RStan: the R interface to Stan. R package
#'   version 2.32.7. \url{https://mc-stan.org/}
#,
"_PACKAGE"

\name{out_beta}
\alias{out_beta}
\docType{data}
\title{
  Estimation results of a Zero Inflated Beta IRT model fit to the Abasement data}
\description{
  The object returned by \code{fitBIRT} containing the results of a zero inflated beta IRT model fit to the Abasement data of the Adjectives Check List (ACL; Gough & Heilbrun, 1980). The Abasement scale has been analyzed using a zero-one inflated beta IRT model by Molenaar et al. (2022). See Table 10 for the model fit statistics of all ACL subscales including the Abasement subscale and see Table 11 for the parameter estimates of the Abasement scale. Note that the estimates in \code{out_beta} and the subsequent model fit statistics obtained using \code{bridgeBIRT} are based on a zero inflated model only, therefore, results are slightly different.}

\format{
A \code{BoundIRT} object obtained using

\code{set.seed(1310)}

\code{data(Abasement)}

\code{out_beta=fitBIRT(Abasement,model="beta")}
}

\references{
Gough, H. G., & Heilbrun, A. B. (1980). \emph{The adjective check list, manual 1980 Edition}.
Consulting Psychologists Press.

Molenaar, D., Cúri, M., & Bazán, J.L. (2022). Zero and One Inflated Item Response Theory Models for Bounded Continuous Data. \emph{Journal of Educational and Behavioral Statistics}, \bold{47}, 693-735.
\doi{10.3102/10769986221108455}
}


\seealso{
\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{loglikBIRT}} to calculate the log-likelihood for each person and each posterior sample. Needed to calculate WAIC and LOOIC for a given model.

\code{\link{repBIRT}} to simulate replicated datasets using (a subset of) the posterior parameter samples. For posterior predictive model checking.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

\code{\link{rBIRT}} simulate data according to a (zero/one inflated) bounded IRT model.
}


\examples{

data(out_beta)

}
\keyword{datasets}

\name{Abasement}
\alias{Abasement}
\alias{female}
\docType{data}
\title{
  Abasement data}
\description{
  Bounded continuous items responses of 244 subjects to the 10 items from the Abasement subscale of the Adjectives Check List (ACL; Gough & Heilbrun, 1980) including the covariate \code{female} which is code \code{0} and  \code{1}}

\format{
The item scores have originally been scored on a 60 milimeter line segment on which the subjects had to indicate to what degree each adjective applies to them. These scores have been recoded into a [0,1] interval. The Abasement scale does not contain 1 scores but it does contain 0 scores.
All subscales of the ACL including the Abasement scale have been analyzed using bounded continuous IRT models with zero-one inflation by Molenaar et al. (2022; see Table 10 for the model fit statistics of all ACL subscales and see Table 11 for the parameter estimates of the Abasement scale). The covariate \code{female} needs to be loaded seperately but subjects are in the same order so that data can be \code{cbind}-ed if needed.}
\references{
Gough, H. G., & Heilbrun, A. B. (1980). \emph{The adjective check list, manual 1980 Edition}.
Consulting Psychologists Press.

Molenaar, D., Cúri, M., & Bazán, J.L. (2022). Zero and One Inflated Item Response Theory Models for Bounded Continuous Data. \emph{Journal of Educational and Behavioral Statistics}, \bold{47}, 693-735. \doi{10.3102/10769986221108455}
}


\seealso{
\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

\code{\link{rBIRT}} simulate data according to a (zero/one inflated) bounded IRT model.
}


\examples{

data(Abasement)
data(female)

}
\keyword{datasets}

\name{repBIRT}
\alias{repBIRT}

\title{
  Simulate replicated datasets using the posterior parameter samples from a BoundIRT-object}
\description{
  To facilitate posterior predictive model checking, this function can be used to simulate replicated datasets for each of the samples from the posterior parameter distribution in a \code{BoundIRT}-object.}

\usage{
repBIRT(object, nrep=1000, silent=FALSE)

}
\arguments{

  \item{object}{A \code{BoundIRT}-object}
  \item{nrep}{Number of replicated datasets to produce. See \bold{Details}.}
  \item{silent}{If TRUE no details are displayed about the progress}
}

\details{
  \code{repBIRT} is a wrapper around \code{rBIRT} extracting the posterior parameter samples, and simulating data for a subset of the samples. That is, for many posterior predictive purposes, not all posterior parameter samples are needed. Therefore, only the last \code{nrep} samples will be used. Setting \code{nrep} to be equal to the total number of posterior samples will use all of them.}


\value{
  An array of size \code{nrep} by \code{N} by \code{nit} containing the replicated datasets.
}
\references{
Gabry J, Simpson D, Vehtari A, Betancourt M, Gelman A (2019). Visualization in
  Bayesian workflow. \emph{Journal of the Royal Statistical Society, A}, \bold{182}, 389-402. \doi{10.1111/rssa.12378}

}
\author{ Dylan Molenaar \email{d.molenaar@uva.nl}}

\seealso{

\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{loglikBIRT}} to calculate the log-likelihood for each person and each posterior sample. Needed to calculate WAIC and LOOIC for a given model.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

}

\examples{
 #load the Abasement data and fitting results for the Beta IRT model
 data(Abasement)
 data(out_beta)

 #generate 10 replicated datasets
 z_rep=repBIRT(out_beta,nrep=10)

 #these can be used for e.g., posterior predictive checks
 #using the suggested bayesplot library
 if (requireNamespace("bayesplot", quietly = TRUE)) {
   bayesplot::ppc_dens_overlay(Abasement[,1],z_rep[,,1])  #for item 1
   bayesplot::ppc_dens_overlay(Abasement[,1],z_rep[,,2])  #for item 2, etc.
 }

}
\keyword{models}
\keyword{multivariate}

\name{loglikBIRT}
\alias{loglikBIRT}

\title{
  Calculate the log-likelihood values of each person for each posterior parameter sample}
\description{
  To facilitate model fit checking (e.g., leave one out cross-validation, WAIC), this function can be used to calculate the log-likelihood values of each person for each posterior parameter sample in a \code{BoundIRT}-object.}

\usage{
loglikBIRT(object, marginal=TRUE,nquad=15, silent=FALSE)

}
\arguments{

  \item{object}{A \code{BoundIRT}-object}
  \item{marginal}{If \code{TRUE} the marginal loglikelihood is calculated for each subject (default), otherwise the conditional likelihood is calculated for each subject-item combination. See \bold{Details}.}
  \item{nquad}{If \code{marginal=TRUE} this argument determines the number of quadrature points used to approximate the integral in the marginal likelihood.}
  \item{silent}{If TRUE no details are displayed about the progress}
}

\details{
For latent variable models like the ones in \code{BoundIRT}, Merkle et al., (2019) recommend the use of the likelihood function that is marginalized over the persons parameters (i.e., not over the item parameters) in calculating fit statistics like the WAIC and LOOIC. Using \code{marginal=TRUE} will output this marginal likelihood value for each person and each sample of the posterior. If the interest is in calculating the conventional fit measures (e.g., conditional WAIC and DIC), \code{marginal=FALSE} should be used to obtain the likelihood values for each person conditional on the person and item parameters of each posterior sample.
  }


\value{
  The function returns a matrix of dimensions (\code{nsamples}) by (\code{N}) containing the (marginal) log-likelihood values of the subjects for each posterior sample.).
}
\references{
Gabry J, Simpson D, Vehtari A, Betancourt M, Gelman A (2019). Visualization in
  Bayesian workflow. \emph{Journal of the Royal Statistical Society, A}, \bold{182}, 389-402. \doi{10.1111/rssa.12378}

Merkle EC, Furr D, Rabe-Hesketh S. (2019).Bayesian Comparison of Latent Variable Models: Conditional Versus Marginal Likelihoods. \emph{Psychometrika}, \bold{84}(3), 802-829. \doi{10.1007/s11336-019-09679-0}

}
\author{ Dylan Molenaar \email{d.molenaar@uva.nl}}

\seealso{

\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{repBIRT}} to simulate replicated datasets using (a subset of) the posterior parameter samples. For posterior predictive model checking.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

}

\examples{
 #read in the prefitted results from the Abasement data (see ?Abasement and ?out_beta)
 data(out_beta)

 # Calculate marginal log-likelihood
 \donttest{
   logL=loglikBIRT(out_beta)

   #logL can now be used for e.g.,model fit assessment using loo() from R-package loo

   #library loo should be available as it is part of rstan which is an import of BoundIRT
   if (requireNamespace("loo", quietly = TRUE)) {
     library(loo)
     loo(logL)   # gives the marginal LOOIC
     waic(logL) # gives the marginal WAIC
 }
 }
 }
\keyword{models}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BoundIRT-package.R
\docType{package}
\name{BoundIRT-package}
\alias{BoundIRT}
\alias{BoundIRT-package}
\alias{boundIRT}
\title{The 'BoundIRT' package.}
\description{
Bounded continuous data are encountered in many areas of test application.
Examples include visual analogue scales used in the measurement of personality, mood,
depression, and quality of life; item response times from tests with item deadlines;
confidence ratings; and pain intensity ratings. Using this package, item response theory (IRT)
models suitable for bounded continuous item scores can be fitted to data within a Bayesian framework.
The package draws on posterior sampling facilities provided by R-package 'rstan' (Stan Development Team, 2025).
Available models include the Beta IRT model by Noel and Dauvier (2007), the continuous response
model by Samejima (1973), the unbounded normal model by Mellenbergh (1994),
and the Simplex IRT model by Flores et al. (2020). All models can be
fitted with or without zero-one inflation (Molenaar et al., 2022).
Model fit comparisons can be conducted using the Watanabe–Akaike information criterion (WAIC),
the leave-one-out cross-validation information criterion (LOOIC), and the fully marginalized likelihood
(i.e., Bayes factors).
}
\references{
Flores, S., Bazan, J.L., & Bolfarine, H. (2020). A hierarchical
joint model for bounded response time and response accuracy. In M. Wiberg,
D. Molenaar, J. González,U. Bockenholt, & K. S. Kim (Eds.), \emph{Quantitative
psychology: The 84th Annual Meeting of the Psychometric Society, Santiago de
Chile, Chile} (pp. 95-109). Springer. \doi{10.1007/978-3-030-43469-4_8}

Mellenbergh, G. J. (1994). A unidimensional latent trait model for continuous
item responses. \emph{Multivariate Behavioral Research}, \bold{29}(3), 223-236.
\doi{10.1207/s15327906mbr2903_2}

Molenaar, D., Cúri, M., & Bazán, J.L. (2022). Zero and One Inflated Item Response
Theory Models for Bounded Continuous Data. \emph{Journal of Educational and
Behavioral Statistics}, \bold{47}, 693-735. \doi{10.3102/10769986221108455}

Noel, Y., & Dauvier, B. (2007). A beta item response model for continuous
bounded responses. \emph{Applied Psychological Measurement}, \bold{31}(1),
47-73. \doi{10.1177/0146621605287691}

Samejima, F. (1973). Homogeneous case of the continuous response model. \emph{Psychometrika},
\bold{38}(2), 203-219.\doi{10.1007/BF02291114}

Stan Development Team (2025). RStan: the R interface to Stan. R package
version 2.32.7. \url{https://mc-stan.org/}
}
\author{
\strong{Maintainer}: Dylan Molenaar \email{d.molenaar@uva.nl}

}

\name{plot.BoundIRT}
\alias{plot.BoundIRT}

\title{
  Plotting function for \code{BoundIRT}-objects.}
\description{
  This function can be used to produce item and test information curves, and the model implied densities of a fitted bounded continuous IRT model.
}

\usage{
\method{plot}{BoundIRT}(x,
     what=c("icc","iic","tic","density"),
     items=NULL,
     separate=FALSE,
     range=c(-3,3),
     by=.001,
     ymax=NULL,
     xlab=NULL,
     ylab=NULL,
     main=NULL,
     col=NULL,
     lwd_prob=2,
     ...)
}
\arguments{

  \item{x}{A \code{BoundIRT}-object to produce the plots for.}

    \item{what}{ A character string indicating what plot to produce. The options are:
      \describe{
              \item{"icc"}{(Default) Item characteristic curves.}
              \item{"iic"}{Item information curves.}
              \item{"tic"}{Test information curves.}
              \item{"density"}{The model implied density for the observed data.}
        }}
    \item{items}{A numeric vector indicating which items to consider for the plots. If NULL all items are plotted (default). If \code{items} is provided and \code{what="tic"}, the test information is based only on these items.}
    \item{separate}{If \code{separate=TRUE} the different items are depicted in separate plots}
    \item{range}{Range of the latent variable on the x-axis. Argument will be neglected for \code{what="density"}.}
    \item{by}{increment for the x-axis of the plot.}
    \item{ymax}{Specify the maximum value for the y-axis. For multiple items, this can be an array of size length(items), or a single value      for all items.}
    \item{xlab}{Specify an alternative label for the x-axis.}
    \item{ylab}{Specify an alternative label for the y-axis.}
    \item{main}{Specify an alternative title for plot.}
    \item{col}{The color code to be used for plotting different items. Can be an array and will be recycled if length(items)!=length(col). Argument can be any valid R color specification. Default is 1:6.}
    \item{lwd_prob}{For \code{what="density"} and a model with zero and/or one inflation, the probabilities of zero and one scores are depicted using vertical lines. The width of these lines can be set using this argument. Default is 2. Width of the other lines can be set using \code{...} below.}
    \item{...}{Other arguments to be passed to \code{\link{matplot}}}.

  }

\details{
  Plotting the model implied densities is relatively slow due to the numerical integration involved in marginalizing over the latent variable. In the presence of zero and/or one inflation, model implied densities inclulde the probability of zero and/or one scores by a vertical line. As this vertical line represents a probability and the rest of the plot is about density, the zero/one probabilities may be masked if density values are large even if the zero/one inflation is quite substantial. See \bold{Examples} how to extract the zero/one probabilities in the case these are hard to see from the plot.
}
\value{
  A matrix containing x-axis values (either the latent variable or the observed variable) and the y-axis values for each item.}

\author{ Dylan Molenaar \email{d.molenaar@uva.nl}}

\seealso{
\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{loglikBIRT}} to calculate the log-likelihood for each person and each posterior sample. Needed to calculate WAIC and LOOIC for a given model.

\code{\link{repBIRT}} to simulate replicated datasets using (a subset of) the posterior parameter samples. For posterior predictive model checking.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

\code{\link{rBIRT}} simulate data according to a (zero/one inflated) bounded IRT model.
}
\examples{

 #load results for zero inflated beta IRT model fit to the Abasement data
 # see ?Abasement and ?out_beta
 data(out_beta)

 # plots for item 1
 oldpar=par(mfrow=c(2,2))
 plot(out_beta,what="icc",items=1)
 plot(out_beta,what="iic",items=1)
 plot(out_beta,what="tic",items=1)
 den_plot=plot(out_beta,what="density",items=1)
 par(oldpar)

 #extract zero probabilities (there are no ones in the Abasement data)
 den_plot$out[1]        # zero probabilities

}
\keyword{models}
\keyword{multivariate}

\name{fitBIRT}
\alias{fitBIRT}

\title{
  Fit various IRT models for bounded continuous items to data}
\description{
  This function fits various IRT models for bounded continuous items using MCMC sampling in R-package 'rstan' (Stan Development Team, 2025).
Available models include the Beta IRT model by Noel and Dauvier (2007), the continuous response
model by Samejima (1973), the unbounded normal model by Mellenbergh (1994), and the Simplex IRT model
by Flores et al. (2020). All models can be fitted with or without a zero and/or one inflation (Molenaar et al., 2022).}

\usage{
fitBIRT(z,
        model=c("beta","samejima","simplex","normal"),
        inflated=c("auto","zero","one","both","none"),
        iter=2000,
        warmup=NULL,
        nchains=1,
        prior=NULL,
        silent=FALSE,
        ...)
}
\arguments{

  \item{z}{A matrix of size \code{N} by \code{n} containing the bounded continuous item scores, where \code{N} is
        the number of subjects and \code{n} is the number of items. The items should be coded in a \eqn{(0,1)}, a \eqn{[0,1)}, a \eqn{(0,1]}, or \eqn{[0,1]} interval.}

    \item{model}{A character string indicating the model to be fit. The options are:
        \describe{
              \item{"beta"}{(Default) The Beta IRT model by Noel and Dauvier (2007) with item specific discrimination parameter.}
              \item{"samejima"}{The IRT model by Samejima (1973).}
              \item{"simplex"}{The Simplex IRT model by Flores et al. (2020).}
              \item{"normal"}{The (unbounded) unidimensional normal linear factor model by Mellenbergh (1994)}
        }}
    \item{inflated}{A character string indicating the kind of inflation to use. The options are:
        \describe{
              \item{"auto"}{(Default) Zero and one inflation parameters are only added for items that actually contain zero and/or one scores               .}
              \item{"zero"}{Zero inflation parameters are estimated for all items.}
              \item{"one"}{One inflation parameters are estimated for all items.}
              \item{"both"}{Zero and one inflation parameters are estimated for all items.}
              \item{"none"}{No zero-one inflation. The data shouldn't contain 0 or 1 scores.}
        }
  }
  \item{iter}{Number of MCMC samples to draw, including warmup samples}
  \item{warmup}{Number of warmup samples. Default is floor(iter/2)}
  \item{nchains}{Number of MCMC chains to use. Default is 1.}
  \item{prior}{If \code{NULL}, the default priors are being used. Otherwise a list can be specified giving the prior standard deviation of (some of) the parameters. See \bold{Details}}
  \item{silent}{If TRUE no details are displayed during estimation}

  \item{...}{Additional arguments to pass to \code{rstan::sampling()}}

  }

\details{
  Function \code{fitBIRT} fits the models discussed by Molenaar et al. (2022) including the facilities for zero and/or one inflation. If \eqn{\alpha_{i}}, \eqn{\beta_{i}}, and \eqn{\phi_{i}} are respectively the discrimination, easiness, and dispersion parameters of item \code{i}, \eqn{\theta_{p}} is the latent variable position of subject \code{p}, and \eqn{z_{pi}} is the bounded continuous item score of subject \code{p} on item \code{i}, then the different models can be given as follows:

  \describe{
  \item{\bold{Beta model}}{

    \eqn{z_{pi} \sim \text{Beta} (a_{pi},b_{pi})}

    \eqn{a_{pi} = \exp(.5(\alpha_{i}\theta_{p}+\beta_{i} + \log(\phi_i)))}

    \eqn{b_{pi} = \exp(.5(-\alpha_{i}\theta_{p}-\beta_{i} + \log(\phi_i)))}
  }
  \item{\bold{Samejima model}}{

    \eqn{z_{pi}^{'} \sim \text{Normal} (\alpha_{i}\theta_{p}+\beta_{i},\phi_i)}

    \eqn{z_{pi}^{'} = \text{logit}(z_{pi})}
 }
  \item{\bold{Simplex model}}{

    \eqn{z_{pi} \sim \text{Simplex}(1/(1+\exp(-(\alpha_{i}\theta_{p}+\beta_{i}))),\phi_i)}
}
  \item{\bold{Normal model}}{

    \eqn{z_{pi} \sim \text{Normal} (\alpha_{i}\theta_{p}+\beta_{i},\phi_i)}
}
  \item{\bold{Zero and/or one inflation}}{

  To these models, zero and/or one inflation can be added as follows:

  \eqn{A = P(z_{pi}=0|\theta_{p}) = 1/(1+\exp(-(\gamma_{0i}-\alpha_{i}\theta_{p})))}

  \eqn{B = P(z_{pi}=1|\theta_{p}) = 1-1/(1+\exp(-(\gamma_{1i}-\alpha_{i}\theta_{p})))}

  where \eqn{\gamma_{0i}} and \eqn{\gamma_{1i}} model the amount of zero and one inflation respectively. The conventional models above are accommodated as follows:

  \eqn{f(z_{pi}|\theta_{p}) = A} for \eqn{z_{pi}=0}

  \eqn{f(z_{pi}|\theta_{p}) = B} for \eqn{z_{pi}=1}

  \eqn{f(z_{pi}|\theta_{p}) = [ (1-B) - A ] \times k(z_{pi}|\theta_{p})} for \eqn{z_{pi}\in(0,1)}


  where \eqn{k(.)} is the distribution according to the conventional model above. See Molenaar et al. (2022) for details.

  In estimating the models above, normal priors are specified for \eqn{\theta_p, \log(\alpha_i), \beta_i, \log(\phi_i), \gamma_{0i}, \text{and } \gamma_{1i}}. All priors have mean 0, and by default, a standard deviation of 1 for \eqn{\theta_p} (for identification purposes) and 10 for the other parameters. Changing (one of) the defaults involves specifying a list with elements \code{sT, sA, sB, sP} and/or \code{sG}, which overrides the default. It is thus possible to stick to the defaults, but change the prior standard deviation for \eqn{\log\alpha_i} and  \eqn{\beta_i} by using for instance \code{prior=list(sA=5,sB=5)}. Note that setting \code{sG} affects both the prior of \eqn{\gamma_{0i} \text{ and } \gamma_{1i}}.
}
}
}
\value{
  An object of class \code{BoundIRT} with values:
   \item{ output}{The posterior means and standard deviations of the item parameters.}
   \item{ theta}{The posterior means and standard deviations of the person parameters.}
   \item{ mcmc}{ The \code{stanfit} object containing all sampling results. See \code{\link[rstan:stanfit-class]{stanfit}}.}
    \item{ prior}{ The prior standard deviations used in the list format discussed above.}

For the \code{BoundIRT} object, the following methods are available:
  \item{summary.BoundIRT}{which gives the default \code{rstan} summary overview of the parameters.}
  \item{plot.BoundIRT}{which provides various IRT plotting facilities, see \code{\link{plot.BoundIRT}}}
  \item{print.BoundIRT}{which gives the posterior means and standard deviations of the item parameters.}
  \item{coef.BoundIRT}{which returns the posterior means of  the item parameters, see \code{\link{coef.BoundIRT}}.}
}
\references{


Flores, S., Bazan, J.L., & Bolfarine, H. (2020). A hierarchical joint model for bounded
response time and response accuracy. In M. Wiberg, D. Molenaar, J. González,
U. Bockenholt, & K. S. Kim (Eds.), \emph{Quantitative psychology: The 84th Annual
Meeting of the Psychometric Society, Santiago de Chile, Chile} (pp. 95-109). Springer.
\doi{10.1007/978-3-030-43469-4_8}


Mellenbergh, G. J. (1994). A unidimensional latent trait model for continuous item
responses. \emph{Multivariate Behavioral Research}, \bold{29}(3), 223-236.
\doi{10.1207/s15327906mbr2903_2}

Molenaar, D., Cúri, M., & Bazán, J.L. (2022). Zero and One Inflated Item Response Theory Models for Bounded Continuous Data. \emph{Journal of Educational and Behavioral Statistics}, \bold{47}, 693-735.
\doi{10.3102/10769986221108455}

Noel, Y., & Dauvier, B. (2007). A beta item response model for continuous bounded
responses. \emph{Applied Psychological Measurement}, \bold{31}(1), 47-73.
\doi{10.1177/0146621605287691}

Samejima, F. (1973). Homogeneous case of the continuous response model. \emph{Psychometrika},
\bold{38}(2), 203-219.\doi{10.1007/BF02291114}

Stan Development Team (2025). RStan: the R interface to Stan. R package version 2.32.7.
\url{https://mc-stan.org/}


}
\author{ Dylan Molenaar \email{d.molenaar@uva.nl}}

\seealso{

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{loglikBIRT}} to calculate the log-likelihood for each person and each posterior sample. Needed to calculate WAIC and LOOIC for a given model.

\code{\link{repBIRT}} to simulate replicated datasets using (a subset of) the posterior parameter samples. For posterior predictive model checking.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

\code{\link{rBIRT}} simulate data according to a (zero/one inflated) bounded IRT model.
}
\examples{
 #load the Abasement data which contains zero inflation
 data(Abasement)

 # fit a zero inflated beta IRT model
 # for illustrative purposes, we only use 20 iterations.
 # In practice use many more!

 res=fitBIRT(Abasement,model="beta",iter=20)
 print(res)   # quick overview
 coef(res)   # extract posterior item parameter means
 summary(res,c("alpha","beta"))   # request the rstan summary


}
\keyword{models}
\keyword{multivariate}

\name{bridgeBIRT}
\alias{bridgeBIRT}

\title{
  Estimate fully marginalized log-likelihood  using bridge sampling.}
\description{
  This function uses bridge sampling as discussed by Gronau et al. (2017) to obtain the fully marginalized likelihood of an estimated bounded IRT model.
}

\usage{
bridgeBIRT(object,niter=10000,tol=1e-7,silent=FALSE)
}
\arguments{

  \item{object}{A \code{BoundIRT}-object.}
  \item{niter}{Number of iterations of the algorithm.}
  \item{tol}{Tolerance for the algorithm. Algorithm stops if the change in \eqn{\log(r_1)} between subsequent iterations is smaller than this value, see \bold{Details}.}
  \item{silent}{If TRUE no progress is displayed during estimation.}
}

\details{
In bridge sampling, a so-called bridge function is used to estimate the fully marginalized likelihood by combining existing posterior samples with additional samples drawn from a proposal distribution. In \code{bridgeBIRT}, a multivariate normal proposal distribution is used. Using these proposal samples, the final expression for the marginal likelihood involves an iterative procedure (see Equation 4.1 from Meng & Wong, 1996) which stops if the change in marginal likelihood between iterations does not exceed the tolerance.

To improve numerical stability and avoid overflow in the \code{exp()} function, we follow the approach described in Appendix B of Gronau et al. (2017). Specifically, the median log-likelihood value, \eqn{l^\star}, is subtracted from the log-marginal likelihood, and only the remaining component, \eqn{r_1}, is estimated. After the iterative scheme has converged, these two components are recombined to obtain the final estimate of the marginal likelihood.
}
\value{
  A list containing:
  \item{log_ml}{The fully marginalized log-likelihood.}
  \item{r1}{The estimate of \eqn{r_1}.}
  \item{l_start}{The value of \eqn{l^\star} used (median log-likelihood across posterior samples).}
  }

\references{
Gronau, Q. F., et al. (2017). A tutorial on bridge sampling. \emph{Journal of mathematical psychology}, \bold{81}, 80-97.
\doi{10.1016/j.jmp.2017.09.005}


Meng, X. L., & Wong, W. H. (1996). Simulating ratios of normalizing constants via a simple identity: a theoretical exploration. \emph{Statistica Sinica}, \bold{6}, 831-860. \url{https://www.jstor.org/stable/24306045}
}

\author{Dylan Molenaar \email{d.molenaar@uva.nl}}

\seealso{

\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

\code{\link{rBIRT}} simulate data according to a (zero/one inflated) bounded IRT model.
}
\examples{

  \donttest{
  # read in the pre-estimated object containing the Abasement results
  # for the Beta IRT model and estimate the marginal likelihood
  \donttest{
  data(out_beta)
  bridgeBIRT(out_beta)
  }
  }

}
\keyword{models}
\keyword{multivariate}

\name{latregBIRT}
\alias{latregBIRT}

\title{
  Latent regression of the latent variable from a bounded IRT model on observed predictor variables}
\description{
  This function takes a fitted \code{BoundIRT} object and subjects the posterior latent variable distribution to a linear regression model.
}

\usage{
latregBIRT(formula, X, object, method=c("posterior","Croon"), nboot=1000)}
\arguments{

  \item{formula}{A formula to specify the linear regression of the latent variable on the predictor variables in \code{X}. See \bold{Examples}.}

    \item{X}{A matrix of predictor variables. The column names of the matrix should at least include the predictor names provided in \code{formula}. All other variables are ignored.}

    \item{object}{A \code{BoundIRT} object obtained with \code{fitBIRT()}.}
    \item{method}{The method to conduct the latent regression. Default is "posterior" which uses the posterior latent variable draws. Other option is "Croon" and "CroonSE" which uses the correction by Croon (2002). See \bold{Details}.}
    \item{nboot}{The number of bootstrap samples for method="Croon". See \bold{Details}.}

  }

\details{
  By default (\code{method="posterior"}), \code{latregBIRT} fits a linear regression model to the posterior latent variable distribution from a bounded continuous IRT model:

  \eqn{\theta=\bold{x\gamma}+\epsilon}

  where \eqn{\bold{x}} is a vector of predictors variables, \eqn{\bold{\gamma}} is a vector of regression parameters, and \eqn{\epsilon} is a vector of residuals. Using the existing draws from the posterior latent variable distribution, \eqn{\tilde{\theta}} in \code{BoundIRT.object}, samples are drawn from the posterior of \eqn{\bold{\gamma}} conditional on \eqn{\tilde{\theta}}. This posterior is normal with its mean being equal to the least square estimate of \eqn{\bold{\gamma}}, and its variance equal to \eqn{V\sigma^2} where \eqn{V} is the inverse of \code{cov(X)}, and \eqn{\sigma^2} is the residual variance which is sampled from it's scaled inverse-chi-square posterior distribution. Note that the posterior of the latent variable is misspecified with respect to the predictor variables. The posterior is however unbiased for the number of items approaching infinity. As most applications of (bounded continuous) IRT models typically involve a limited number of items, the posterior covariance matrix of the latent variable and the predictor variables is adjusted for the nonzero variance in the latent variable posterior. The correction is based on the two-step approach by Croon (2002) in which the factor scores from a linear factor model are estimated in step 1 and used in a regression model in step 2. As such, Croon's method is a statistically valid alternative to fitting the factor model and latent regression model simultaneously. Vermunt (2025) discusses how Croon's method can be generalized to models with heterogenous measurement error like IRT models. \code{latregBIRT} implements this idea by correcting the covariance matrix of the latent variable and the predictor variables for each posterior latent variable draw using the formula from Vermunt (2025). Next, a linear regression is fit to each sample as described above.

For method="Croon" the above correction is applied on the EAP latent variable scores. Drawback is that no standard deviation corrections are available. These are therefore bootstrapped using \code{nboot} resamples.
}
\value{
  \item{output}{A matrix containing the postrior means and standard deviations of the regression slopes for each predictor (if method="posterior") or the regression slope estimates with boostrapped standard errors (if method="Croon").}
  \item{samples}{A matrix containing the samples from the postrior distribution of the regression parameter (if method="posterior") or the regression parameter estimates on the bootstrapped samples (if method="Croon").}
}
\references{

Croon, M. (2002). Using predicted latent scores in general latent structure models. In G. Marcoulides and I. Moustaki (Eds.), \emph{Latent Variable and Latent Structure Models} (pp. 195-224). Lawrence Erlbaum.

Vermunt J.K. (2025). Stepwise estimation of latent variable models: An overview of approaches. \emph{Statistical Modelling}, \bold{25}(6),530-551. \doi{10.1177/1471082X251355693}

}
\author{ Dylan Molenaar \email{d.molenaar@uva.nl}}

\seealso{

\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{loglikBIRT}} to calculate the log-likelihood for each person and each posterior sample. Needed to calculate WAIC and LOOIC for a given model.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{rBIRT}} simulate data according to a (zero/one inflated) bounded IRT model.
}
\examples{

 # open pre-estimated Beta IRT model on Abasement data (see ?Abasement)
  data(out_beta)

 # simulate a latent regression with 2 covariates

 EAP=out_beta$theta[,1]
 N=length(EAP)
 age=.4*EAP+sqrt(1-.4^2)*rnorm(N)
 ses=.3*EAP+sqrt(1-.3^2)*rnorm(N)

 # conduct latent regression
 predictors=cbind(age,ses)
 latregBIRT(~age+ses,predictors,out_beta)

}
\keyword{models}
\keyword{multivariate}

\name{rBIRT}
\alias{rBIRT}

\title{
  Simulate data according to various IRT models for bounded continuous items}
\description{
  This function simulates data according to various IRT models for bounded continuous items.
Available models include the Beta IRT model by Noel and Dauvier (2007), the continuous response
model by Samejima (1973), the unbounded normal model by Mellenbergh (1994), and the Simplex IRT model
by Flores et al. (2020). All models can be simulated  with or without zero and/or one inflation (Molenaar et al., 2022).}

\usage{
rBIRT(N,nit,alpha,beta,phi,theta,
      gamma0=NULL,
      gamma1=NULL,
      model=c("beta","samejima","simplex","normal"))

}
\arguments{

  \item{N}{Number of subjects}
  \item{nit}{Number of items}
  \item{alpha}{A nit-dimensional array of true discrimination parameter values}
  \item{beta}{A nit-dimensional array of true easiness parameter values}
  \item{phi}{A nit-dimensional array of true dispersion parameter values}
  \item{theta}{A N-dimensional array of true person parameter values}
  \item{gamma0}{Optional: A nit-dimensional array of true zero inflation parameters}
  \item{gamma1}{Optional: A nit-dimensional array of true one inflation parameters}

    \item{model}{A character string indicating what model to use to simulate the data. The options are:
        \describe{
              \item{"beta"}{(Default) The Beta IRT model by Noel and Dauvier (2007) with item specific discrimination parameter.}
              \item{"samejima"}{The IRT model by Samejima (1973).}
              \item{"simplex"}{The Simplex IRT model by Flores et al. (2020).}
              \item{"normal"}{The (unbounded) unidimensional normal linear factor model by Mellenbergh (1994)}
        }}
     }

\details{
  If arguments gamma0 and gamma1 are not provided, data is simulated according to the conventional models. Zero and one inflation can be introduced seperately by only providing gamma0 or gamma1, or zero and one inflation can both be introduced by specifying both gamma0 and gamma1. If the interest is in simulating some -but not all- items with zero and/or one inflation, the uninflated items can be coded as -Inf in gamma0 and Inf in gamma1. See \link{fitBIRT} for the exact parameterization of the models.}


\value{
  A matrix of size \code{N} by \code{nit} containing the simulated item responses.
}
\references{



Flores, S., Bazan, J.L., & Bolfarine, H. (2020). A hierarchical joint model for bounded
response time and response accuracy. In M. Wiberg, D. Molenaar, J. González,
U. Bockenholt, & K. S. Kim (Eds.), \emph{Quantitative psychology: The 84th Annual
Meeting of the Psychometric Society, Santiago de Chile, Chile} (pp. 95-109). Springer.
\doi{10.1007/978-3-030-43469-4_8}


Mellenbergh, G. J. (1994). A unidimensional latent trait model for continuous item
responses. \emph{Multivariate Behavioral Research}, \bold{29}(3), 223-236.
\doi{10.1207/s15327906mbr2903_2}

Molenaar, D., Cúri, M., & Bazán, J.L. (2022). Zero and One Inflated Item Response Theory Models for Bounded Continuous Data. \emph{Journal of Educational and Behavioral Statistics}, \bold{47}, 693-735.
\doi{10.3102/10769986221108455}

Noel, Y., & Dauvier, B. (2007). A beta item response model for continuous bounded
responses. \emph{Applied Psychological Measurement}, \bold{31}(1), 47-73.
\doi{10.1177/0146621605287691}

Samejima, F. (1973). Homogeneous case of the continuous response model. \emph{Psychometrika},
\bold{38}(2), 203-219.\doi{10.1007/BF02291114}

Stan Development Team (2025). RStan: the R interface to Stan. R package version 2.32.7.
\url{https://mc-stan.org/}

}
\author{ Dylan Molenaar \email{d.molenaar@uva.nl}}

\seealso{

\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{loglikBIRT}} to calculate the log-likelihood for each person and each posterior sample. Needed to calculate WAIC and LOOIC for a given model.

\code{\link{repBIRT}} to simulate replicated datasets using (a subset of) the posterior parameter samples. For posterior predictive model checking.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

}

\examples{

 # data sim with 100 subjects and 10 items according to
 # beta IRT model with zero and one inflation

 nit=10
 N=100
 beta=runif(nit,.5,1)
 alpha=runif(nit,.5,.7)
 gamma0=runif(nit,-3,-1.5)
 gamma1=runif(nit,1.5,3)

 theta=rnorm(N)

 phi=seq(1.5,2.5,length=nit)
 z=rBIRT(N,nit,alpha,beta,phi,theta,gamma0,gamma1,model="beta")

}
\keyword{models}
\keyword{multivariate}

\name{coef.BoundIRT}
\alias{coef.BoundIRT}

\title{
  Obtain item parameter estimates}
\description{
  Extracts a matrix of posterior means for the item parameters of a \code{BoundIRT}-object}

\usage{
\method{coef}{BoundIRT}(object,sd=FALSE, ...)}

\arguments{

  \item{object}{A \code{BoundIRT}-object.}

    \item{sd}{A Boolean indicating if posterior standard deviations should be returned as well}
    \item{...}{Other arguments to be passed to \code{print()}}
    }

\value{
  A matrix containing the posterior mean (and standard deviations if sd=TRUE) of the item parameters from a bounded IRT model fit with the \code{fitBIRT()} function.}

\author{ Dylan Molenaar \email{d.molenaar@uva.nl}}

\seealso{
\code{\link{fitBIRT}} to estimate various bounded continuous IRT models to data.

\code{\link{plot.BoundIRT}} to plot the item and test information curves and predicted densities for the different models.

\code{\link{coef.BoundIRT}} to extract item parameter estimates from a \code{BoundIRT} object.

\code{\link{bridgeBIRT}} to use bridge sampling to calculate the marginal loglikelihood for a model in a \code{BoundIRT} object.

\code{\link{latregBIRT}} to conduct a latent regression of the person parameters on some observed predictor variables.

\code{\link{rBIRT}} simulate data according to a (zero and/or one inflated) bounded IRT model.

}
\examples{

  # read in the pre-estimated object containing the Abasement results
  data(out_beta)

  coef(out_beta)


}
\keyword{models}
\keyword{multivariate}
